%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Predictors-based Multi-horizon Stochastic Discount Factor Bounds with
% Parameter Uncertainty
%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% June, 2017   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% -------------------------------------------------------------------------
% This code investigate how we account for uncertainty in the evaluation
% of the difference between the estimated model-implied standard deviation of the SDF
% and predictors-based variance bounds.
% All the computation steps are report in the Online Appendicx, Part C, based on the method by
% Cecchetti, Lam, and Mark (1994) and Burnside (1994).
%
% SET 1: stock market + 3 month treasury government bill [roll-over strategy]
% 
% SET 5: SET A + Value Growth portfolios (new SET B in the revised version for JBES)
% -------------------------------------------------------------------------
% 1. We only use real returns for all the assets in this excercise
% 2. This exercise studies 1-quarter 1-year and 5-year SDF bounds properties
% -------------------------------------------------------------------------
% 
%% Initial Setting
clear all; clc; close all;
addpath('.\Library\gmm;','.\Results\HJBounds','.\Library','.\data',...
    '.\Library\minz;','.\Library\HJBLibrary','.\Library\Library_CHJ')
addpath('.\Results\simulation','.\Results\HJBounds')

SET = 1;                % 1: SET A
                        % 5: SET E (new SET B in the revised version  for JBES, with also middle)

load longrun5Portfolios.mat   % 1947Q2-2012Q4
cstr=cal(1947,2,4,255); 
ss = ical(1952,2,cstr);       % sample size
se = ical(2012,3,cstr);

rmkt = r(ss:se,6); rPtf = r(ss:se,1:5); % real log return, 
% rPtf is for Fama-French 5-portfolio formed by BM ratio 
% [growth portfolio with low BM(return) and value portfolio with high BM(return)]
Rmkt=exp(rmkt);   RPtf = exp(rPtf);  Rrf =exp(rrf(ss:se));     % real gross return

lpd = log(QPD(ss+1:se+1,6));
InflationLong = Inflation(1:end-1); Inflation = Inflation(ss:se);

load  cay_q_12Q3.txt
cay = cay_q_12Q3(2:end,5);

load Pred_stock;
load Pred_bond;

% nominal value-growth portfolio returns and return predictors (See folder EqPtfsForFOTY)
load EqPtfsForFOTY; % norminal portfolio returns and return predictors
R_PtfReal = R_Ptf - repmat(Inflation,1,size(R_Ptf,2));

%% Fama-Bliss bond yields - using for constructing spread
% load bond price data
% this is the file from CRSP completely unmodified
% sample: 1952-06 - 2012-12
% We compute 1-year holding period return with different maturity FB discount bond

load Fama_Bliss_price.txt
bondprice = Fama_Bliss_price;
T=length(bondprice); %(2012-1952+1)*12 -5 -3 

y=-log(bondprice(:,2:end)/100).*(ones(T,1)*[1/1 1/2 1/3 1/4 1/5]);
famablisyld=[bondprice(:,1) y];
yields=famablisyld(:,2:end);
mats=[1 2 3 4 5]'; 
prices=-(ones(T,1)*mats').*yields;

keep Rmkt Rrf lpd Inflation SET InflationLong cay yields Pred_stock Pred_bond Pred_Ptf R_Ptf R_PtfReal

%% US Government Bonds 
%% loaded from Indices/Deciles: Treasury and Inflation -- US Treasury and Inflation - Monthly

load .\data\bondreturns19252012.txt;

cstr=cal(1925,12,12,size(bondreturns19252012,1)); 
ss  = ical(1952,6,cstr); se = ical(2012,12,cstr);

bondret30yQ=chgfreq(bondreturns19252012(:,2),3,1,0);
bondret30yQ=bondret30yQ(ss:3:se-3);

bondret20yQ=chgfreq(bondreturns19252012(:,3),3,1,0);
bondret20yQ=bondret20yQ(ss:3:se-3);

bondret10yQ=chgfreq(bondreturns19252012(:,4),3,1,0);
bondret10yQ=bondret10yQ(ss:3:se-3);

bondret7yQ=chgfreq(bondreturns19252012(:,5),3,1,0);
bondret7yQ=bondret7yQ(ss:3:se-3);

bondret5yQ=chgfreq(bondreturns19252012(:,6),3,1,0);
bondret5yQ=bondret5yQ(ss:3:se-3);

bondret2yQ=chgfreq(bondreturns19252012(:,7),3,1,0);
bondret2yQ=bondret2yQ(ss:3:se-3);

bondret1yQ=chgfreq(bondreturns19252012(:,8),3,1,0);
bondret1yQ=bondret1yQ(ss:3:se-3);

% the spread is the difference between the log yield on a 5-year artificial zero-coupon bond from the CRSP Fama-Bliss Discount Bond File, and the log yield on the Treasury Bill
TB1M = bondreturns19252012(ss:se,10); 
spr  = yields(:,5)/12-TB1M; sprQ=spr(1:3:end-3);

TB1M = chgfreq(bondreturns19252012(:,10),3,1,0); %The short rate is the log yield on the 30-day Treasury Bill from CRSP
TB1M = TB1M(ss:3:se-3);

% Roll over 3M Tbill returns
TB3MSretQ = chgfreq(bondreturns19252012(:,9),3,1,0);
TB3MSretQ = TB3MSretQ(1:3:end-3);

cstr=cal(1925,4,4,size(TB3MSretQ,1)); 
ss  = ical(1947,2,cstr); se = ical(2012,3,cstr);
TB3MSretQ = TB3MSretQ(ss:se);  

% realized roll over 3M Tbill returns
TB3MSretQReal   = TB3MSretQ - InflationLong;
TB3MSroll1YReal = chgfreq(TB3MSretQReal,4,1,0);
TB3MSroll5YReal = chgfreq(TB3MSretQReal,4*5,1,0);     %THIS IS NOT ANNUALIZED

cstr=cal(1947,2,4,size(TB3MSretQ,1)); 
ss  = ical(1952,2,cstr);       
se  = ical(2012,3,cstr);
TB3MrollReal = [TB3MSretQReal(ss:se) TB3MSroll1YReal(ss:se) TB3MSroll5YReal(ss:se)];

% realized rate of government bonds with different maturities
bondret1yQ_real  = 1 + bondret1yQ -Inflation;
bondret2yQ_real  = 1 + bondret2yQ -Inflation;
bondret5yQ_real  = 1 + bondret5yQ -Inflation;
bondret7yQ_real  = 1 + bondret7yQ -Inflation;
bondret10yQ_real = 1 + bondret10yQ-Inflation;
bondret20yQ_real = 1 + bondret20yQ-Inflation;
bondret30yQ_real = 1 + bondret30yQ-Inflation;

%% Predictive Regressions of Returns
horizons = [1 4 5*4];        % 1-quarter, 1-year and 5-year

if  SET   == 1
    assetret = NaN(size(Rmkt,1),2,size(horizons,2));
elseif SET ==5
    assetret = NaN(size(Rmkt,1),10,size(horizons,2)); 
end

load Pred_Reg % load predictive regression results with estimated parameters and standard deviations 

%% Predictive Regression of Asset Returns
for hh=1:length(horizons)
    horizon = horizons(hh); frequency = 1; offset = 0;
    y1     = exp(trimr(chgfreq(log(Rmkt),horizon,frequency,offset),horizon,0));
    y2     = exp(trimr(TB3MrollReal(:,hh),horizon,0));
    yb1y   = exp(trimr(chgfreq(log(bondret1yQ_real) ,horizon,frequency,offset),horizon,0));
    yb2y   = exp(trimr(chgfreq(log(bondret2yQ_real) ,horizon,frequency,offset),horizon,0));
    yb5y   = exp(trimr(chgfreq(log(bondret5yQ_real) ,horizon,frequency,offset),horizon,0));
    yb7y   = exp(trimr(chgfreq(log(bondret7yQ_real) ,horizon,frequency,offset),horizon,0));
    yb10y  = exp(trimr(chgfreq(log(bondret10yQ_real),horizon,frequency,offset),horizon,0));
    yb20y  = exp(trimr(chgfreq(log(bondret20yQ_real),horizon,frequency,offset),horizon,0));
    yb30y  = exp(trimr(chgfreq(log(bondret30yQ_real),horizon,frequency,offset),horizon,0));
    
    % Value-Growth portfolio returns
    yp1    = exp(trimr(chgfreq(log(R_PtfReal(:,1)),horizon,frequency,offset),horizon,0));
    yp2    = exp(trimr(chgfreq(log(R_PtfReal(:,2)),horizon,frequency,offset),horizon,0));
    yp3    = exp(trimr(chgfreq(log(R_PtfReal(:,3)),horizon,frequency,offset),horizon,0));
    yp4    = exp(trimr(chgfreq(log(R_PtfReal(:,4)),horizon,frequency,offset),horizon,0));
    yp5    = exp(trimr(chgfreq(log(R_PtfReal(:,5)),horizon,frequency,offset),horizon,0));
    yp6    = exp(trimr(chgfreq(log(R_PtfReal(:,6)),horizon,frequency,offset),horizon,0));
    yp7    = exp(trimr(chgfreq(log(R_PtfReal(:,7)),horizon,frequency,offset),horizon,0));
    yp8    = exp(trimr(chgfreq(log(R_PtfReal(:,8)),horizon,frequency,offset),horizon,0));
    %% %%%%%%%%%%%%%%%%%%%%%%%%    
    
    smpl_range = size(y1,1);
    x1    = [ones(smpl_range,1) trimr(Pred_stock,0,horizon)];
    xb    = [ones(smpl_range,1) trimr(Pred_bond,0,horizon)];
    % ---------------------------------------------------------------------

    % regressors of Value Growth Portfolios
    xp1   = [ones(smpl_range,1) trimr(Pred_Ptf(:,1),0,horizon) trimr(Pred_stock,0,horizon)];
    xp2   = [ones(smpl_range,1) trimr(Pred_Ptf(:,2),0,horizon) trimr(Pred_stock,0,horizon)];
    xp3   = [ones(smpl_range,1) trimr(Pred_Ptf(:,3),0,horizon) trimr(Pred_stock,0,horizon)];
    xp4   = [ones(smpl_range,1) trimr(Pred_Ptf(:,4),0,horizon) trimr(Pred_stock,0,horizon)];
    xp5   = [ones(smpl_range,1) trimr(Pred_Ptf(:,5),0,horizon) trimr(Pred_stock,0,horizon)];
    xp6   = [ones(smpl_range,1) trimr(Pred_Ptf(:,6),0,horizon) trimr(Pred_stock,0,horizon)];
    xp7   = [ones(smpl_range,1) trimr(Pred_Ptf(:,7),0,horizon) trimr(Pred_stock,0,horizon)];
    xp8   = [ones(smpl_range,1) trimr(Pred_Ptf(:,8),0,horizon) trimr(Pred_stock,0,horizon)];    

    % ---------------------------------------------------------------------    
    %%  seemingly unrelated regression estimates         
    if SET ==1
        neqs = 2;
        y(1).eq = y1;    x(1).eq= x1;
        y(2).eq = y2;    x(2).eq= xb;
        assetret(1+horizon:end,:,hh) = [y1 y2];
        result = sur(neqs,y,x);
        beta_stock(hh,:)= result(1).beta'; beta_Tbill(hh,:)=result(2).beta';
        cov_stock_Tbill(:,:,hh)  = result.sigma;
    elseif SET == 5
        neqs = 10;
        y(1).eq = y1;    x(1).eq = x1;
        y(2).eq = y2;    x(2).eq = xb;
        y(3).eq = yp1;   x(3).eq = xp1;
        y(4).eq = yp2;   x(4).eq = xp2;
        y(5).eq = yp3;   x(5).eq = xp3;
        y(6).eq = yp4;   x(6).eq = xp4;
        y(7).eq = yp5;   x(7).eq = xp5;
        y(8).eq = yp6;   x(8).eq = xp6;
        y(9).eq = yp7;   x(9).eq = xp7;
        y(10).eq = yp8;   x(10).eq = xp8;
        assetret(1+horizon:end,:,hh) = [y1 y2 yp1 yp2 yp3 yp4 yp5 yp6 yp7 yp8];  
        result = sur(neqs,y,x);
        beta_stock(hh,:)    = result(1).beta'; beta_Tbill(hh,:)    =result(2).beta';
        beta_ptf1(hh,:)     = result(3).beta'; beta_ptf2(hh,:)     =result(4).beta';
        beta_ptf3(hh,:)     = result(5).beta'; beta_ptf4(hh,:)     =result(6).beta';
        beta_ptf5(hh,:)     = result(7).beta'; beta_ptf6(hh,:)     =result(8).beta';
        beta_ptf7(hh,:)     = result(9).beta'; beta_ptf8(hh,:)     =result(10).beta';
        cov_stock_Tbill_Ptf(:,:,hh)  = result.sigma;        
    end
    
    if SET == 1
        mu_ret = [result(1).yhat  result(2).yhat  ];
        resid  = [result(1).resid result(2).resid ];
    elseif SET == 5
        mu_ret = [result(1).yhat  result(2).yhat result(3).yhat result(4).yhat result(5).yhat result(6).yhat result(7).yhat result(8).yhat result(9).yhat result(10).yhat];
        resid  = [result(1).resid result(2).resid result(3).resid result(4).resid result(5).resid result(6).resid result(7).resid result(8).resid result(9).resid result(10).resid];
    end
    E_m          = 0.3:0.0005:2.5;
    sig2_ret   = result.sigma;
    sig2_BL(hh,:)         = OSB_BL_Robust( E_m',assetret(1+horizon:end,:,hh),mu_ret,sig2_ret);

end

%% %% Draw all the parameters from a normal distribution
% Draw from a normal with mean = estimated parameters and variance =  the diagonal of the variance-covariance matrix (coming from predictive regressions)
NSIM = 10000;
seed = 17234;
rand('seed', seed)
randn('seed', seed)

if SET == 1
    numPhi = 2*3;   
elseif SET == 5
    numPhi = 2*3+8*4;
end
Phi    = NaN (NSIM,numPhi,size(horizons,2));

sim_E_m1Q    = 0.5:0.0005:1.2;  % save some time for calculation
sim_E_m1Y    = 0.5:0.0005:1.5; 
sim_E_m5Y    = 0.3:0.0005:2;  

% sim_sig2_BL  = NaN(size(E_m,1),NSIM,size(horizons,2));

for hh = 1:size(horizons,2)
    if SET == 1
        Phi_mean    = [beta_stock(hh,:) beta_Tbill(hh,:)];
        SIG_Phi     = [se_beta_stock(hh,:) se_beta_Tbill(hh,:)];
    elseif SET == 5
        Phi_mean    = [beta_stock(hh,:) beta_Tbill(hh,:) beta_ptf1(hh,:) beta_ptf2(hh,:)...
                       beta_ptf3(hh,:) beta_ptf4(hh,:) beta_ptf5(hh,:) beta_ptf6(hh,:) beta_ptf7(hh,:) beta_ptf8(hh,:)];
        SIG_Phi     = [se_beta_stock(hh,:) se_beta_Tbill(hh,:) se_beta_Ptf(hh,:,1)/5 se_beta_Ptf(hh,:,2)/5 ...
                       se_beta_Ptf(hh,:,3)/5 se_beta_Ptf(hh,:,4)/5 se_beta_Ptf(hh,:,5)/5 se_beta_Ptf(hh,:,6)/5 ...
                       se_beta_Ptf(hh,:,7)/5 se_beta_Ptf(hh,:,8)/5];      
    end
    
    e           = randn(NSIM,numPhi);
    for ii = 1:numPhi
        if hh == 3
            Phi(:,ii,hh) = Phi_mean(ii) + e(:,ii).*SIG_Phi(ii)/5;
%             Phi(:,ii,hh) = Phi_mean(ii) + e(:,ii).*SIG_Phi(ii)/10;
        elseif hh == 1
            Phi(:,ii,hh) = Phi_mean(ii) + e(:,ii).*SIG_Phi(ii)/6;
%             Phi(:,ii,hh) = Phi_mean(ii) + e(:,ii).*SIG_Phi(ii)/10;
        elseif hh == 2
            Phi(:,ii,hh) = Phi_mean(ii) + e(:,ii).*SIG_Phi(ii)/8;
%             Phi(:,ii,hh) = Phi_mean(ii) + e(:,ii).*SIG_Phi(ii)/10;
        end
    end
end

%% Compute Variance Bound with Estimated Parameter Uncertainty
for sim =1:10
    sim
    for hh = 1:size(horizons,2)     
        horizon = horizons(hh); frequency = 1; offset = 0;
        x1 = trimr(Pred_stock,0,horizon); x2 = trimr(Pred_bond,0,horizon); xp = trimr(Pred_Ptf,0,horizon);
        x1 = [ones(size(x1,1),1) x1];     x2 = [ones(size(x2,1),1) x2];    xp = [ones(size(xp,1),1) xp trimr(Pred_stock,0,horizon)];
%         b1 = Phi(sim,1:size(x1,2),hh);    b2 = Phi(sim,size(x1,2)+1:size(x1,2)+size(x2,2),hh);
        if SET == 1
            b1 = Phi(sim,[1:3],hh);    b2 = Phi(sim,[4:6],hh); 
            mu_ret = [x1*b1' x2*b2'];
            sig2_ret = cov_stock_Tbill(:,:,hh);
        elseif SET == 5
            b1 = Phi(sim,[1:3],hh); b2 = Phi(sim,[4:6],hh); b3 = Phi(sim,[7:10],hh); b4 = Phi(sim,[11:14],hh);
            b5 = Phi(sim,[15:18],hh); b6 = Phi(sim,[19:22],hh); b7 = Phi(sim,[23:26],hh); b8 = Phi(sim,[27:30],hh);
            b9 = Phi(sim,[31:34],hh); b10 = Phi(sim,[35:38],hh);
            mu_ret = [x1*b1' x2*b2' xp(:,[1 2 10 11])*b3' xp(:,[1 3 10 11])*b4' xp(:,[1 4 10 11])*b5' xp(:,[1 5 10 11])*b6' xp(:,[1 6 10 11])*b7' xp(:,[1 7 10 11])*b8' xp(:,[1 8 10 11])*b9' xp(:,[1 9 10 11])*b10'];
            sig2_ret = cov_stock_Tbill_Ptf(:,:,hh);
        end
        if hh == 1
            sim_sig2_BL1Q(:,sim)  = OSB_BL_Robust(sim_E_m1Q',assetret(1+horizon:end,:,hh),mu_ret,sig2_ret);
        elseif hh == 2
            sim_sig2_BL1Y(:,sim)  = OSB_BL_Robust(sim_E_m1Y',assetret(1+horizon:end,:,hh),mu_ret,sig2_ret);
        elseif hh == 3
            sim_sig2_BL5Y(:,sim)  = OSB_BL_Robust(sim_E_m5Y',assetret(1+horizon:end,:,hh),mu_ret,sig2_ret);
        end
        clear mu_ret sig2_ret
    end
end

fname = [cd, '\Results\simulation\CLM_BL_Bound_SET',num2str(SET)];
save(fname,'sim_sig2_BL1Q','sim_E_m1Q','sim_sig2_BL1Y','sim_E_m1Y','sim_sig2_BL5Y','sim_E_m5Y');

% % % fname = [cd, '\data\Pred_Reg' ];
% % % save(fname, 'Pred_stock','Pred_bond','beta_stock','se_beta_stock','beta_Tbill','se_beta_Tbill','cov_stock_Tbill','Rmkt','TB3MrollReal');

figure 
plot(sim_E_m1Q,sqrt(sim_sig2_BL1Q));hold on; plot(E_m, sqrt(sig2_BL(1,:)),'k');xlim([0.95,1.05]);ylim([0,2]); 
figure 
plot(sim_E_m1Y,sqrt(sim_sig2_BL1Y));hold on; plot(E_m, sqrt(sig2_BL(2,:)),'k');xlim([0.9,1.07]);ylim([0,2.5]);
figure 
plot(sim_E_m5Y,sqrt(sim_sig2_BL5Y));hold on; plot(E_m, sqrt(sig2_BL(3,:)),'k');xlim([0.6,1.2]);ylim([0,4]);
