%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% CLM test applied to Long-Run Risk Models
% 
% December, 2017 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. We simulate the LLR model by using both estimated and calibrated
%    parameters
% 2. The estimated parameters are taken from Bansal, Kiku and Yaron (2016,JME), 
%    RISKS FOR THE LONG RUN: ESTIMATION WITH TIME AGGREGATION
% 3. The calibrated parameters are takn from Bansal, Kiku and Yaron (2009), 
%    An Empirical Evaluation of the Long Run Risks Models for Asset Prices
% 4. calculate the estimated standard deviation of mu_SDF (1Q, 1Y and 5Y) and std_SDF
% 5. 10 simulations with 600,000 months
% 6. reference: Cecchetti, Lam and Mark (1994, JF)
%
% REMARK: Set estimate = 0, the model is simulated by using calibrated parameters. 
%         Set estimate = 1, the model is simulated with estimated parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;

addpath('.\Library','.\Library\Library_LRR','.\data','.\Library.\Library_ConfidenceBands','.\Results\HJBounds','.\Results\simulation')
addpath('.\Library\gmm;','.\Results\HJBounds','.\Library','.\data','.\Library\minz;','.\Library\HJBLibrary','.\Library\Library_CHJ')

             
burnin = 240;

% step 1
NSIM = 10000;
T    = 742; 

%%
if T == 600000
    seed = 17234;
    rand('seed', seed)
    randn('seed', seed)
else
    seed = [758 19568 20254 13548 51549 65 65000 17234];
end
%%
global mu rho vu sigma sigmaw phie gamma psi delta corr_dc
global phi pi_d phid mud A0 A2 zbar k1 beta_ae stochvol

stochvol=1;         % 1: stochastic volatility
                    % 0: constant volatility
            
% The model parameters are based on the process given in equation

% The parameters are given in monthly terms

%% Model Parameter Setting

%% estimated parameters and SE, (Bansal (2016, JME table 2))
%preference
gamma=9.67;    sig_gamma = 1.44;
psi  =2.18;    sig_psi   = 0.21;
delta=0.9990;  sig_delta = 0.0001;

freq_LR = 11;  sig_freq_LR = 2.16;  %decision periods per year
%mean consumption growth
mu = 0.0016;   sig_mu = 0.0005;
%mean dividend growth
mud= 0.0027;   sig_mud = 0.0010;

theta= (1-gamma)/(1-1/psi);

%Persistence
rho=0.9762;    sig_rho = 0.0035;
%DIV VOL multiple
phid=4.65;     sig_phid = 0.48;
%VOL multiple
phie=0.0318;   sig_phie = 0.0053;
%Leverage
phi=4.51;      sig_phi = 0.45;
%Vol Persistence
vu=0.9984;     sig_vu = 0.0007;
%Vol of Vol
sigmaw=2.12*10^(-6); sig_sigmaw = 5.32*10^(-7);
%Base SD
sigma=0.0070;  sig_sigma = 0.0009;
%
corr_dc = 0.51;sig_corr_dc = 0.10;
%Consumption Exposure
pi_d= 0.0;

totcensoring=0;

%% Load results related with variance bounds
SET  =1;           % 1: SET A: stock market + risk free bond 
                   % 2: SET B: stock market + risk free bond + 5-,7-, 10-, 20-, 30-year government bonds       
                   % 4: SET D: SET A + Value Growth portfolios
                   % 5: SET D: SET A + Value, Middle and Growth portfolios (new SET B in the revised version for JBES)
if SET == 1
    load SDF_MH_SET1
    load CLM_BL_Bound_SET1 % load the results of predictor-based variance bound with parameter uncertainty
elseif SET == 5
    load SDF_MH_SET5
    load CLM_BL_Bound_SET5     
end

load Pred_Reg % estimation results from predictive regressions of asset returns
Em = E_m;

          
%% MODEL SIMULATION  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Em = 0.2:0.001:2;   % range of the mean of SDFs (consistenct with our bootstrap range)
horizon     = [3 12 60]; % 1-q, 1-y and 5-year holding

%% Load the previous simulation results to get mu_vFIXED
load sim_LRRBKY2016_NSIM10_T600000
mu_vFIXED = mean(simmean_c(2:10,:),1);

%% Draw all the parameters from a normal distribution
Beta        = [mu rho phie sigma vu sigmaw mud phi phid corr_dc];
numBeta     = size(Beta,2);
SIG_Beta    = diag([sig_mu sig_rho sig_phie sig_sigma sig_vu sig_sigmaw...
                    sig_mud sig_phi sig_phid sig_corr_dc].^2);

for ss = 1:size(seed,2)   

    rand('seed', seed(ss))
    randn('seed', seed(ss))

    randnNum    = randn(NSIM,numBeta);
    for ii = 1:numBeta
        Beta_sim(:,ii) = Beta(ii) + randnNum(:,ii).*sqrt(SIG_Beta(ii,ii));
    end
    
    count1 = 1; count2 = 1; count3 = 1;

for sim = 1:NSIM
    mu = Beta_sim(sim,1);       
    if Beta_sim(sim,2)<1 
        rho = Beta_sim(sim,2); 
    else
        rho = Beta(2); 
    end;
    phie = Beta_sim(sim,3);
    sigma = Beta_sim(sim,4); 
    if Beta_sim(sim,5)<1
        vu = Beta_sim(sim,5);
    else
        vu = Beta(5);
    end
    sigmaw = Beta_sim(sim,6);
    mud = Beta_sim(sim,7);      
    phi = Beta_sim(sim,8);      
    phid = Beta_sim(sim,9);
    if Beta_sim(sim,10)>0 & Beta_sim(sim,10) < 1
        corr_dc =Beta_sim(sim,10);
    else
        corr_dc = Beta(10);
    end
    
    %% Solve for price consumption ratio
    %      zbar = fixed_point_pricecons(p0, N);
    tol = 1e-05; maxit=100;
    p0=3; % initial value
    [k,zbar,err,P] = fixpt('pricecons',p0,tol,maxit);
    
    k1 = exp(zbar)/(1+exp(zbar)); A1 = (1-1/psi)/(1-k1*rho);
    k0 = log(1.0 + exp(zbar))-k1*zbar;
    beta_ae = k1*A1*phie;
    if stochvol==1
        A2 = 0.5*((theta - theta/psi)^2+(theta*beta_ae)^2)/(theta*(1-k1*vu));
    else
        A2 = 0;
    end
    beta_aw = k1*A2;
    
    A0 = (log(delta) + mu*(1-1/psi) + k0 +  beta_aw * sigma^2 * (1-vu) + ...
        0.5*theta* (beta_aw*sigmaw)^2) /(1-k1);
    %      plotyy(1:516,xt,1:516,pc)
    
    %% Solve for price dividend ratio
    [k,zpdbar,err,P] = fixpt('pricediv',p0,tol,maxit); clear p0
    
    k1m = exp(zpdbar)/(1+exp(zpdbar)); k0m = log(1.0 + exp(zpdbar)) - k1m*zpdbar;
    
    A1m = (phi-1/psi)/(1-k1m*rho);
    beta_me = k1m*A1m*phie;
    
    lambda_e =  (1-theta)*beta_ae;
    lambda_w =  (1-theta)*A2*k1;
    lambda_n = -(- theta/psi + theta - 1); %== gamma
    if stochvol==1
        A2m = ((1-theta)*A2*(1-k1*vu)+0.5*((pi_d - lambda_n)^2+(beta_me-lambda_e)^2 + phid^2))/((1-k1m*vu));
    else
        A2m = 0;
    end
    beta_mw = k1m*A2m;
    
    A0m = (theta*log(delta) + mu*theta*(1-1/psi) - lambda_w*(sigma^2)*(1-vu) + ...
        (theta-1)*( k0 + A0*(k1-1) ) - mu + ...
        k0m +  beta_mw * sigma^2 * (1-vu) + ...
        + mud + 0.5*(beta_mw-lambda_w)^2*(sigmaw)^2) /(1-k1m);
    
    %% Simulation Results
    fprintf('Iteration \t\t: %d \n',sim);
    
    %% Generate 4 sets of i.i.d. N(0,1)
    zeta = zeros(burnin+T,3);
    %%
%     seed = 17234;
%     rand('seed', seed)
%     randn('seed', seed)
    e = randn(burnin+T,4);
    %%
    gt1=zeros(burnin+T,1);
    xt =zeros(burnin+T,1);
    vol2=repmat(sigma^2,burnin+T,1);
    gdt1=zeros(burnin+T,1);
    
    for i=1:burnin+T-1
        if stochvol ==1
            vol2(i+1,1) = sigma^2 + vu*(vol2(i,1) - sigma^2 )+ sigmaw*e(i+1,1);
        else
            vol2(i+1,1) = sigma^2;
        end
        if vol2(i+1,1) < 0
            vol2(i+1)=0.0001*sigma^2;
            totcensoring=totcensoring+1;
        end
        gt1(i+1,1) =mu  +     xt(i,1)  +        sqrt(vol2(i,1)) * e(i+1,2);
        gdt1(i+1,1)=mud + phi*xt(i,1)  + phid * sqrt(vol2(i,1)) * e(i+1,3) + pi_d * sqrt(vol2(i,1)) * e(i+1,2); %In the BY model \pi_d = 0 so BY solutions are a special case.
        xt(i+1,1)  =      rho*xt(i,1)  + phie * sqrt(vol2(i,1)) * e(i+1,4);
        
        zeta(i+1,1) = sqrt(vol2(i,1)) * e(i+1,2);
        zeta(i+1,2) = sqrt(vol2(i,1)) * e(i+1,4);
        zeta(i+1,3) = sigmaw*e(i+1,1);
    end
    xt  = trimr(xt,burnin,0) ; vol2 = trimr(vol2,burnin,0);
    gt1 = trimr(gt1,burnin,0); gdt1 = trimr(gdt1,burnin,0);
    
    zeta = trimr(zeta,burnin,0);
    % plot([gt1 xt])
    
    log_pc_ratio=A0 +A1 *xt+A2 *vol2;
    log_pd_ratio=A0m+A1m*xt+A2m*vol2;
    
    rretMkt  = k0m +  k1m*log_pd_ratio(2:end) + gdt1(2:end,1)- log_pd_ratio(1:end-1);
    rretCons = k0  +  k1 *log_pc_ratio(2:end) + gt1(2:end,1) - log_pc_ratio(1:end-1);
    
    %Consumption claim
    var_ra  =(beta_ae^2 + 1)*vol2(1:end-1,1) +(beta_aw*sigmaw)^2; %Annualized Market Volatility sqrt(mean(var_rm)*12)*100
    ConsumptionRP=lambda_n * 1 * vol2(1:end-1,1) + beta_ae*lambda_e    *vol2(1:end-1,1) + beta_aw*lambda_w*sigmaw^2 - 0.5*var_ra;
    %Dividend claim
    var_rm  =(beta_me^2 + phid^2)*vol2(1:end-1,1) +(beta_mw*sigmaw)^2; %Annualized Market Volatility sqrt(mean(var_rm)*12)*100
    MarketRP     =lambda_n * pi_d * vol2(1:end-1,1) + beta_me*lambda_e *vol2(1:end-1,1) + beta_mw*lambda_w*sigmaw^2 - 0.5*var_rm;
    %% BY (2004) risk-free rate
    Et_gt1   = mu + xt(1:end-1,1);
    vart_mt1 = (lambda_n^2+lambda_e^2)*vol2(1:end-1,1)+lambda_w^2*sigmaw^2;
    rrf_by   = -log(delta) + (1/psi)*Et_gt1 + (1-theta)/theta*ConsumptionRP - (1/(2*theta))*vart_mt1;
    %% Gallant risk-free rate
    Gamma0_1 = log(delta) - (1/psi)*mu;
    Gamma0_2 = -0.5*theta*(theta - 1)*(k1*A2*sigmaw)^2;
    Gamma0 = Gamma0_1 + Gamma0_2;
    Gamma1 = -1/psi;
    Gamma2 = (theta - 1)*(k1*vu - 1)*A2;
    Y = zeros(size(rretMkt,1),3);
    Y(:,1) = ones(size(rretMkt,1),1);
    Y(:,2) = xt(1:end-1,1);
    Y(:,3) = vol2(1:end-1,1);
    rrf_g = -Gamma0-Gamma1*Y(:,2)-Gamma2*Y(:,3)-0.5*lambda_n^2*vol2(1:end-1,1)- 0.5*lambda_e^2*vol2(1:end-1,1)-0.5*lambda_w^2*sigmaw^2;
    
    %% Pricing Kernel (IMRS)
    log_mrs  = theta*log(delta)-(theta/psi)*gt1(2:end,1)+(theta-1)*rretCons;
    MRS      = exp(log_mrs);
    
    %% Construct 1-Quarter and 5-year MRS from monthly data
    MRS_1Q = exp(chgfreq(log_mrs,horizon(1),horizon(1),horizon(1)-1));   
    MRS_1Y = exp(chgfreq(log_mrs,horizon(2),horizon(2),horizon(2)-1));  
    MRS_5Y = exp(chgfreq(log_mrs,horizon(3),horizon(3),horizon(3)-1)); 
    
    if T == 742
        if isnan(mean(MRS_1Q))
            coult1= count1;
        else
            sim_MRS_1Q(:,sim) = MRS_1Q;
            coult1= count1+1;
        end
        if isnan(mean(MRS_1Y))
            coult2= count2;
        else
            sim_MRS_1Y(:,sim) = MRS_1Y;
            coult2= count2+1;
        end
        if isnan(mean(MRS_5Y))
            coult3= count3;
        else
            sim_MRS_5Y(:,sim) = MRS_5Y;
            coult3= count3+1;
        end
    end
    
    % Compute the mean, mean[SDF]
    simmean_1Q(sim,ss)   = mean(MRS_1Q,1);   
    simmean_1Y(sim,ss)   = mean(MRS_1Y,1);    
    simmean_5Y(sim,ss)   = mean(MRS_5Y,1);
    
    %% [Random Random] 
    % Compute the standard deviation of simulated SDF, std(SDF)
    simsd_1Q(sim,ss)     = std(MRS_1Q,1);
    simsd_1Y(sim,ss)     = std(MRS_1Y,1);
    simsd_5Y(sim,ss)     = std(MRS_5Y,1);
    % Compute the distance between simulated SDF and simulated BL bound
    [TF1Q  diff1Q_RandRand(sim,ss) sigbound1Q_RandRand(sim,ss)]  = IsItWithinHJB(sim_E_m1Q,sqrt(sim_sig2_BL1Q(:,sim)),simmean_1Q(sim,ss),simsd_1Q(sim,ss));
    [TF1Y  diff1Y_RandRand(sim,ss) sigbound1Y_RandRand(sim,ss)]  = IsItWithinHJB(sim_E_m1Y,sqrt(sim_sig2_BL1Y(:,sim)),simmean_1Y(sim,ss),simsd_1Y(sim,ss));
    [TF5Y  diff5Y_RandRand(sim,ss) sigbound5Y_RandRand(sim,ss)]  = IsItWithinHJB(sim_E_m5Y,sqrt(sim_sig2_BL5Y(:,sim)),simmean_5Y(sim,ss),simsd_5Y(sim,ss));

    %% [Random Fixed] 
    % Compute the distance between simulated SDF and estimated BL bound
    [TF1Q  diff1Q_RandFixed(sim,ss) sigbound1Q_RandFixed(sim,ss)]  = IsItWithinHJB(Em,sqrt(sig2_BL(1,:)),simmean_1Q(sim,ss),simsd_1Q(sim,ss));
    [TF1Y  diff1Y_RandFixed(sim,ss) sigbound1Y_RandFixed(sim,ss)]  = IsItWithinHJB(Em,sqrt(sig2_BL(2,:)),simmean_1Y(sim,ss),simsd_1Y(sim,ss));
    [TF5Y  diff5Y_RandFixed(sim,ss) sigbound5Y_RandFixed(sim,ss)]  = IsItWithinHJB(Em,sqrt(sig2_BL(3,:)),simmean_5Y(sim,ss),simsd_5Y(sim,ss));
    
    %% [Fixed Random] 
    % Compute the standard deviation of SDF, when mu_v is fixed, SDF-Mean(SDF)+mu_vFixed and take its STDEV
    MRS_1Q_vFIXED       = MRS_1Q - (simmean_1Q(sim,ss) - mu_vFIXED(1))*ones(size(MRS_1Q));
    MRS_1Y_vFIXED       = MRS_1Y - (simmean_1Y(sim,ss) - mu_vFIXED(2))*ones(size(MRS_1Y));
    MRS_5Y_vFIXED       = MRS_5Y - (simmean_5Y(sim,ss) - mu_vFIXED(3))*ones(size(MRS_5Y));
    simmean_1Q_vFIXED(sim,ss)   = mean(MRS_1Q_vFIXED,1);
    simmean_1Y_vFIXED(sim,ss)   = mean(MRS_1Y_vFIXED,1);
    simmean_5Y_vFIXED(sim,ss)   = mean(MRS_5Y_vFIXED,1);
    simsd_1Q_vFIXED(sim,ss)     = std(MRS_1Q_vFIXED,1);
    simsd_1Y_vFIXED(sim,ss)     = std(MRS_1Y_vFIXED,1);
    simsd_5Y_vFIXED(sim,ss)     = std(MRS_5Y_vFIXED,1);
    % % Compute the distance between simulated SDF and estimated BL bound when mu_v is fixed
    [TF1Q  diff1Q_FixedRand(sim,ss) sigbound1Q_FixedRand(sim,ss)]  = IsItWithinHJB(sim_E_m1Q,sqrt(sim_sig2_BL1Q(:,sim)),simmean_1Q_vFIXED(sim,ss),simsd_1Q_vFIXED(sim,ss));
    [TF1Y  diff1Y_FixedRand(sim,ss) sigbound1Y_FixedRand(sim,ss)]  = IsItWithinHJB(sim_E_m1Y,sqrt(sim_sig2_BL1Y(:,sim)),simmean_1Y_vFIXED(sim,ss),simsd_1Y_vFIXED(sim,ss));
    [TF5Y  diff5Y_FixedRand(sim,ss) sigbound5Y_FixedRand(sim,ss)]  = IsItWithinHJB(sim_E_m5Y,sqrt(sim_sig2_BL5Y(:,sim)),simmean_5Y_vFIXED(sim,ss),simsd_5Y_vFIXED(sim,ss));    
   
end

end

diff_RandRand       = [reshape(diff1Q_RandRand,ss*NSIM,1) reshape(diff1Y_RandRand,ss*NSIM,1) reshape(diff5Y_RandRand,ss*NSIM,1)];
diff_RandFixed      = [reshape(diff1Q_RandFixed,ss*NSIM,1) reshape(diff1Y_RandFixed,ss*NSIM,1) reshape(diff5Y_RandFixed,ss*NSIM,1)];
diff_FixedRand      = [reshape(diff1Q_FixedRand,ss*NSIM,1) reshape(diff1Y_FixedRand,ss*NSIM,1) reshape(diff5Y_FixedRand,ss*NSIM,1)];

%% Find NaN Values from diff (see IsItWithinHJB.m)
[row_RandRand col_RandRand] = find(isnan(diff_RandRand));
[row_RandFixed col_RandFixed] = find(isnan(diff_RandFixed));
[row_FixedRand col_FixedRand] = find(isnan(diff_FixedRand));

%% Report and Save the Results
mean_diff_RandRand = nanmean(diff_RandRand,1); 
sd_diff_RandRand   = nanstd(diff_RandRand,1);
temp = sort(diff_RandRand);
a = temp(1:79160,1); b = temp(1:78649,2); c = temp(1:65341,3); % 8 seeds
qut_diff_RandRand  = [quantile(a,[0.05 0.5 0.95])' quantile(b,[0.05 0.5 0.95])' quantile(c,[0.05 0.5 0.95])'];
prob_RandRand = [size(find(diff_RandRand(:,1)<0),1) size(find(diff_RandRand(:,2)<0),1) size(find(diff_RandRand(:,3)<0),1)]./size(diff_RandRand,1);
% In Table 3 we report qut_diff_RandRand(3,:)'

mean_diff_RandFixed = nanmean(diff_RandFixed,1);
sd_diff_RandFixed   = nanstd(diff_RandFixed,1);
temp = sort(diff_RandFixed);
a = temp(1:79160,1); b = temp(1:78649,2); c = temp(1:65341,3); % 8 seeds
qut_diff_RandFixed  = [quantile(a,[0.05 0.5 0.95])' quantile(b,[0.05 0.5 0.95])' quantile(c,[0.05 0.5 0.95])'];
prob_RandFixed = [size(find(diff_RandFixed(:,1)<0),1) size(find(diff_RandFixed(:,2)<0),1) size(find(diff_RandFixed(:,3)<0),1)]./size(diff_RandFixed,1);
% In Table 3 we report qut_diff_RandFixed(3,:)'

mean_diff_FixedRand = nanmean(diff_FixedRand,1);
sd_diff_FixedRand   = nanstd(diff_FixedRand,1);
temp = sort(diff_FixedRand);
a = temp(1:79225,1); b = temp(1:79225,2); c = temp(1:79225,3); % 8 seeds
qut_diff_FixedRand  = [quantile(a,[0.05 0.5 0.95])' quantile(b,[0.05 0.5 0.95])' quantile(c,[0.05 0.5 0.95])'];
prob_FixedRand = [size(find(diff_FixedRand(:,1)<0),1) size(find(diff_FixedRand(:,2)<0),1) size(find(diff_FixedRand(:,3)<0),1)]./size(diff_FixedRand,1);
% In Table 3 we report qut_diff_FixedRand(3,:)'

%% Reported Values in Table 3 in the paper
[qut_diff_FixedRand(3,:)' qut_diff_RandFixed(3,:)' qut_diff_RandRand(3,:)' ]

%% Save the results
fname = [cd, '\Results\simulation\CLM_LRRBKY2012_NSIM',num2str(NSIM),'_T',num2str(T),'_SET',num2str(SET),'_8seeds'];
save(fname,'diff_RandRand','diff_RandFixed','diff_FixedRand','simmean_1Q','simmean_1Y','simmean_5Y','simsd_1Q','simsd_1Y','simsd_5Y');

