%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% CLM test applied to Long-Run Risk Models by Schorfheide, Song and Yaron
% 
% October, 2016    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. We simulate the LLR-SSY (Schorfheide, Song and Yaron, 2016) model by using estimated parameters
% 2. calculate the estimated standard deviation of mu_SDF (1Q, 1Y and 5Y) and std_SDF
% 3. 10 simulations with 600,000 months
%
% Remark: we adopte the median of estimated parameter values reported in Table 5 to simulate the model. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;

addpath('.\Library','.\Library\Library_LRR_SSY','.\data','.\Library.\Library_ConfidenceBands','.\Results\HJBounds','.\Results\simulation')
addpath('.\Library\gmm;','.\Results\HJBounds','.\Library','.\data','.\Library\minz;','.\Library\HJBLibrary','.\Library\Library_CHJ')

             
burnin = 240;

% step 1
NSIM = 10000; % 10000
T    = 742;

seed = [758 19568 20254 13548 51549 65 65000 17234];
%%

global delta psi gamma rho rho_lambda sigma_lambda rho_hx rho_hc varphi_c varphi_x sigma sigma_hc sigma_hx mu_c

            
% The model parameters are based on the process given in equation

% The parameters are given in monthly terms

%% Model Parameter Setting

%% estimated parameters and SE

% The parameters are given in monthly terms
% We use the estimated posterior median vaule of all the estimated parameters from
% Schorfheide, Song and Yaron (2016, Table 5)

% Household Preferences
% Household Preferences
delta      = 0.999;     
psi         = 1.93;     sig_psi = (1.93-1.13)/1.64;         % we constructed based on the quantile values in Table 5
gamma  = 8.60;    sig_gamma = (8.60-5.44)/1.64;

theta= (1-gamma)/(1-1/psi);

% Preference Risk
rho_lambda   = 0.956;   sig_rho_lambda = (0.956-0.916)/1.64;
sigma_lambda = 0.0005;  sig_sigma_lambda = (0.0005-0.0003)/1.64;

% Consumption Growth Process
mu_c      = 0.0016; % fixed, see table 5
rho       = 0.987;  sig_rho = (0.987-0.949)/1.64;
varphi_x  = 0.232;  sig_varphi_x = (0.232-0.139)/1.64;
sigma     = 0.0032; sig_sigma = (0.0032-0.0020)/1.64;
rho_hc    = 0.991;  sig_rho_hc = (0.991-0.973)/1.64;
varphi_c  = 1; % fixed in the paper, normalized based on the consumption series
sigma2_hc = 0.0088; sig_sigma2_hc = (0.0088-0.0074)/1.64;
sigma_hc  = sqrt(sigma2_hc); sig_sigma_hc = (sqrt(0.0088)-sqrt(0.0074))/1.64;
rho_hx    = 0.994;  sig_rho_hx = (0.994-0.987)/1.64;
sigma2_hx = 0.0039; sig_sigma2_hx = (0.0039-0.0027)/1.64;
sigma_hx  = sqrt(sigma2_hx); sig_sigma_hx = (sqrt(0.0039)-sqrt(0.0027))/1.64;

% Dividend Growth Process
mu_d      = 0.0010; % fixed, see table 5
phi       = 4.15;   sig_phi = (4.15-2.82)/1.64;
pi        = 1.54;   sig_pi = (1.54-0.204)/1.64;
varphi_d  = 5.02;   sig_varphi_d = (5.02-3.56)/1.64;
rho_hd    = 0.967;  sig_rho_hd = (0.967-0.948)/1.64;
sigma2_hd = 0.0393; sig_sigma2_hd = (0.0393-0.0174)/1.64;
sigma_hd  = sqrt(sigma2_hd); sig_sigma_hd = (sqrt(0.0393)-sqrt(0.0174))/1.64;

% Consumption Measurement Error
sigma_epsilon   = 0.0010;  sig_sigma_epsilon = (0.0010-0.0006)/1.64;
sigma_epsilon_a = 0.0231;  sig_sigma_epsilon_a = (0.0231-0.0061)/1.64; 
 

% measurement error standeard deviations, fixed, see table 5, 10% of the
% sample satandeard deviation of the dividend growth and the risk-free rate
% % % sigma_epsilon_da = 0.1 * ; 
% % % sigma_epsilon_f  = 0.1 * ;

          
%% MODEL SIMULATION  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Em = 0.2:0.001:2;   % range of the mean of SDFs (consistenct with our bootstrap range)
horizon     = [3 12 60]; % 1-q, 1-y and 5-year holding

%% Load the previous simulation results to get mu_vFIXED
load sim_LRRSSY_NSIM10_T600000_SET1
mu_vFIXED = [mean(simmean_1Q) mean(simmean_1Y) mean(simmean_5Y)];

%% Draw all the parameters from a normal distribution
Beta        = [rho_lambda sigma_lambda rho varphi_x sigma rho_hc sigma_hc rho_hx sigma_hx];
numBeta     = size(Beta,2);
SIG_Beta    = diag([sig_rho_lambda sig_sigma_lambda sig_rho sig_varphi_x sig_sigma sig_rho_hc sig_sigma_hc sig_rho_hx sig_sigma_hx].^2);

for ss = 1:size(seed,2)   

    rand('seed', seed(ss))
    randn('seed', seed(ss))

    randnNum    = randn(NSIM,numBeta);
    for ii = 1:numBeta
        Beta_sim(:,ii) = Beta(ii) + randnNum(:,ii).*sqrt(SIG_Beta(ii,ii));
    end
    
    count1 = 1; count2 = 1; count3 = 1;

for sim = 1:NSIM
          
    if Beta_sim(sim,1)<1 
        rho_lambda = Beta_sim(sim,1); 
    else
        rho = Beta(1); 
    end;
    
    if Beta_sim(sim,2)>0
        sigma_lambda = Beta_sim(sim,2);
    else
        sigma_lambda = Beta(2);
    end;
    
    if Beta_sim(sim,3)<0
        rho = Beta_sim(sim,3);
    else
        rho = Beta(3);
    end;
    
    varphi_x = Beta_sim(sim,4);
    
    if Beta_sim(sim,5)>0
        sigma = Beta_sim(sim,5);
    else
        sigma = Beta(5);
    end;    
    
    if Beta_sim(sim,6)<1
        rho_hc = Beta_sim(sim,6);
    else
        rho_hc = Beta(6);
    end;       
    
    if Beta_sim(sim,7)>0
        sigma_hc = Beta_sim(sim,7);
    else
        sigma_hc = Beta(7);
    end;   
    
    if Beta_sim(sim,8)<1
        rho_hx = Beta_sim(sim,8);
    else
        rho_hx = Beta(8);
    end;    
    
    if Beta_sim(sim,9)>0
        sigma_hx = Beta_sim(sim,9);
    else
        sigma_hx = Beta(9);
    end;      
    
    %% Solve for price consumption ratio 
    options = optimset('MaxIter', 1e20, 'MaxFunEvals', 1e20, 'TolFun',1e-20, 'Display', 'off');
    zcfcn   = @(x) z_bar(x);
    zc0     = 3;
%     [zmean] = fsolve(zcfcn,zc0,options);
     zmean = 3;
   
    %% %%%%%%%%%%%%%%%%%%%%%%%%%
    
    k1 = exp(zmean)/(1+exp(zmean)); 
    if k1>0.9999
        k1=0.9999;
    end
    k0 = log(1.0 + exp(zmean))-k1*zmean;
    
    A1 = (1-1/psi)/(1-k1*rho);
    A1_lambda = rho_lambda/(1-k1*rho_lambda);
    
    A2_x = 0.5*theta*(k1*A1)^2/(1-k1*rho_hx);
    A2_c = 0.5*theta*(1-1/psi)^2/(1-k1*rho_hc);
    
    sigma_wc = 2*(varphi_c*sigma)^2*sigma_hc;
    sigma_wx = 2*(varphi_x*sigma)^2*sigma_hx;
    
    A0_1 = log(delta)+k0+mu_c*(1-1/psi)+k1*A2_x*(1-rho_hx)*(varphi_x*sigma)^2 + k1*A2_c*(1-rho_hc)*(varphi_c*sigma)^2;
    A0_2 = 0.5*theta*((k1*A1_lambda+1)^2*sigma_lambda^2+(k1*A2_x*sigma_wx)^2+(k1*A2_c*sigma_wc)^2);
    A0   = (A0_1+A0_2)/(1-k1); 
        
    %% Simulation Results
    fprintf('Iteration \t\t: %d \n',sim);
    
    %% Generate 5 sets of i.i.d. N(0,1)
    eta = randn(burnin+T,3);
    w   = randn(burnin+T,2);
    
    %%
    gt1=zeros(burnin+T,1);
    xt =zeros(burnin+T,1);
    xt_lambda =zeros(burnin+T,1);
    sigma2_ct = zeros(burnin+T,1);  sigma2_ct(1) = sigma^2;
    sigma2_xt = zeros(burnin+T,1);  sigma2_xt(1) = (varphi_x*sigma)^2;
    log_pc_ratio = zeros(burnin+T,1);
    rretCons = zeros(burnin+T,1);
    
    %%
    h_x = zeros(burnin+T,1); 
    h_c = zeros(burnin+T,1);
    sigma_ct = zeros(burnin+T,1);  
    sigma_xt = zeros(burnin+T,1);  
    %%
    
    for i=1:burnin+T-1        
    
        h_c(i+1,1) = rho_hc * h_c(i,1) +sigma_hc*w(i+1,1);
        h_x(i+1,1) = rho_hx *  h_x(i,1) +sigma_hx*w(i+1,2);
        sigma_ct(i+1,1) = varphi_c*sigma*exp(h_c(i+1,1));   sigma2_ct(i+1,1) = sigma_ct(i+1,1)^2;
        sigma_xt(i+1,1) = varphi_x*sigma*exp(h_x(i+1,1));   sigma2_xt(i+1,1) = sigma_xt(i+1,1)^2;
        
        xt(i+1,1)  = rho*xt(i,1)+sqrt(sigma2_xt(i,1)) * eta(i+1,1);
        gt1(i+1,1) = mu_c + xt(i,1) + sqrt(sigma2_ct(i,1)) * eta(i+1,2);
        
        xt_lambda(i+1,1)  = rho_lambda * xt_lambda(i,1) + sigma_lambda * eta(i+1,3);
        
        log_pc_ratio(i+1,1) = A0 + A1 *xt(i+1,1)+ A1_lambda *xt_lambda(i+1,1) + A2_c*sigma2_ct(i+1,1) + A2_x*sigma2_xt(i+1,1);
        rretCons(i+1,1) = k0 + k1*log_pc_ratio(i+1,1) - log_pc_ratio(i,1) + gt1(i+1,1);
    end
    sigma2_ct  = trimr(sigma2_ct,burnin,0) ;
    sigma2_xt  = trimr(sigma2_xt,burnin,0) ;
    xt  = trimr(xt,burnin,0) ; xt_lambda  = trimr(xt_lambda,burnin,0) ;
    gt1 = trimr(gt1,burnin,0); 
    log_pc_ratio = trimr(log_pc_ratio,burnin,0); 
    rretCons = trimr(rretCons,burnin,0);    
    
    %% Pricing Kernel (IMRS)
    log_mrs  = theta*log(delta) -(theta/psi)*gt1 + theta*xt_lambda  +(theta-1)*rretCons;
    MRS      = exp(log_mrs);
    
    %% Construct 1-Quarter and 5-year MRS from monthly data
    MRS_1Q = exp(chgfreq(log_mrs,horizon(1),horizon(1),horizon(1)-1));   
    MRS_1Y = exp(chgfreq(log_mrs,horizon(2),horizon(2),horizon(2)-1));  
    MRS_5Y = exp(chgfreq(log_mrs,horizon(3),horizon(3),horizon(3)-1)); 
    
    if T == 742
        if isnan(mean(MRS_1Q)) | mean(MRS_1Q)>5
            count1= count1;
        else
            sim_MRS_1Q(:,count1,ss) = MRS_1Q;
            simmean_1Q(count1,ss) = mean(MRS_1Q);
            simsd_1Q(count1,ss) = std(MRS_1Q);
            count1= count1+1;
        end
        if isnan(mean(MRS_1Y)) | mean(MRS_1Y)>10
            count2= count2;
        else
             sim_MRS_1Y(:,count2,ss) = MRS_1Y;
            simmean_1Y(count2,ss) = mean(MRS_1Y);
            simsd_1Y(count2,ss) = std(MRS_1Y);
            count2= count2+1;
        end
        if isnan(mean(MRS_5Y)) | mean(MRS_5Y)>15
            count3= count3;
        else
             sim_MRS_5Y(:,count3,ss) = MRS_5Y;
            simmean_5Y(count3,ss) = mean(MRS_5Y);
            simsd_5Y(count3,ss) = std(MRS_5Y);
            count3= count3+1;
        end
    end
end
end

%% Load results related with variance bounds
SET  = 1;          % 1: SET A: stock market + risk free bond 
                   % 2: SET B: stock market + risk free bond + 5-,7-, 10-, 20-, 30-year government bonds        
if SET == 1
    load SDF_MH_SET1
    load CLM_BL_Bound_SET1 % load the results of predictor-based variance bound with parameter uncertainty
elseif SET == 5
    load SDF_MH_SET5
    load CLM_BL_Bound_SET5 % load the results of predictor-based variance bound with parameter uncertainty
end

load Pred_Reg % estimation results from predictive regressions of asset returns
numPhi = size([beta_stock(1,:) beta_Tbill(1,:)],2);
Em = E_m;

%% compute the distance between model-impled SDF and predictors-based bound
for ss = 1:size(seed,2)   
   
for cc = 1:9894
    [TF1Q  diff1Q_RandRand(cc,ss) sigbound1Q_RandRand(cc,ss)]  = IsItWithinHJB(sim_E_m1Q,sqrt(sim_sig2_BL1Q(:,cc)),simmean_1Q(cc,ss),simsd_1Q(cc,ss));
    [TF1Q  diff1Q_RandFixed(cc,ss) sigbound1Q_RandFixed(cc,ss)]  = IsItWithinHJB(Em,sqrt(sig2_BL(1,:)),simmean_1Q(cc,ss),simsd_1Q(cc,ss));
    
    MRS_1Q_vFIXED       = sim_MRS_1Q(:,cc,ss)  - (simmean_1Q(cc,ss) - mu_vFIXED(1))*ones(size( sim_MRS_1Q(:,cc,ss) ));
    simmean_1Q_vFIXED(cc,ss)   = mean(MRS_1Q_vFIXED,1);
    simsd_1Q_vFIXED(cc,ss)     = std(MRS_1Q_vFIXED,1);
    [TF1Q  diff1Q_FixedRand(cc,ss) sigbound1Q_FixedRand(cc,ss)]  = IsItWithinHJB(sim_E_m1Q,sqrt(sim_sig2_BL1Q(:,cc)),simmean_1Q_vFIXED(cc,ss),simsd_1Q_vFIXED(cc,ss));  
end


for cc = 1:9909
    [TF1Y  diff1Y_RandRand(cc,ss) sigbound1Y_RandRand(cc,ss)]  = IsItWithinHJB(sim_E_m1Y,sqrt(sim_sig2_BL1Y(:,cc)),simmean_1Y(cc,ss),simsd_1Y(cc,ss));
    [TF1Y  diff1Y_RandFixed(cc,ss) sigbound1Y_RandFixed(cc,ss)]  = IsItWithinHJB(Em,sqrt(sig2_BL(2,:)),simmean_1Y(cc,ss),simsd_1Y(cc,ss));
    
    MRS_1Y_vFIXED       = sim_MRS_1Y(:,cc,ss)  - (simmean_1Y(cc,ss) - mu_vFIXED(1))*ones(size( sim_MRS_1Y(:,cc,ss) ));
    simmean_1Y_vFIXED(cc,ss)   = mean(MRS_1Y_vFIXED,1);
    simsd_1Y_vFIXED(cc,ss)     = std(MRS_1Y_vFIXED,1);
    [TF1Y  diff1Y_FixedRand(cc,ss) sigbound1Y_FixedRand(cc,ss)]  = IsItWithinHJB(sim_E_m1Y,sqrt(sim_sig2_BL1Y(:,cc)),simmean_1Y_vFIXED(cc,ss),simsd_1Y_vFIXED(cc,ss));
end

for cc = 1:9965
    [TF5Y  diff5Y_RandRand(cc,ss) sigbound5Y_RandRand(cc,ss)]  = IsItWithinHJB(sim_E_m5Y,sqrt(sim_sig2_BL5Y(:,cc)),simmean_5Y(cc,ss),simsd_5Y(cc,ss));
    [TF5Y  diff5Y_RandFixed(cc,ss) sigbound5Y_RandFixed(cc,ss)]  = IsItWithinHJB(Em,sqrt(sig2_BL(3,:)),simmean_5Y(cc,ss),simsd_5Y(cc,ss));
    
    MRS_5Y_vFIXED       = sim_MRS_5Y(:,cc,ss)  - (simmean_5Y(cc,ss) - mu_vFIXED(1))*ones(size( sim_MRS_5Y(:,cc,ss) ));
    simmean_5Y_vFIXED(cc,ss)   = mean(MRS_5Y_vFIXED,1);
    simsd_5Y_vFIXED(cc,ss)     = std(MRS_5Y_vFIXED,1);
    [TF5Y  diff5Y_FixedRand(cc,ss) sigbound5Y_FixedRand(cc,ss)]  = IsItWithinHJB(sim_E_m5Y,sqrt(sim_sig2_BL5Y(:,cc)),simmean_5Y_vFIXED(cc,ss),simsd_5Y_vFIXED(cc,ss));
end
  
count1 = 9894; count2 = 9909; count3 = 9965;
diff_1Q_RandRand =  reshape(diff1Q_RandRand,ss*count1,1);  
diff_1Q_RandFixed =  reshape(diff1Q_RandFixed,ss*count1,1);
diff_1Q_FixedRand =  reshape(diff1Q_FixedRand,ss*count1,1);

diff_1Y_RandRand =  reshape(diff1Y_RandRand,ss*count2,1); 
diff_1Y_RandFixed =  reshape(diff1Y_RandFixed,ss*count2,1);
diff_1Y_FixedRand =  reshape(diff1Y_FixedRand,ss*count2,1);

diff_5Y_RandRand =  reshape(diff5Y_RandRand,ss*count3,1); 
diff_5Y_RandFixed =  reshape(diff5Y_RandFixed,ss*count3,1);
diff_5Y_FixedRand =  reshape(diff5Y_FixedRand,ss*count3,1);

end

temp = sort(diff_1Q_RandRand);  diff_1Q_RandRand = diff_1Q_RandRand(1:79143,1); clear temp
temp = sort(diff_1Q_RandFixed);  diff_1Q_RandFixed = diff_1Q_RandFixed(1:79143,1); clear temp
temp = sort(diff_1Q_FixedRand);  diff_1Q_FixedRand = diff_1Q_FixedRand(1:79143,1); clear temp
diff_1Q = [diff_1Q_RandRand diff_1Q_RandFixed diff_1Q_FixedRand];

temp = sort(diff_1Y_RandRand);  diff_1Y_RandRand = diff_1Y_RandRand(1:77386,1); clear temp
temp = sort(diff_1Y_RandFixed);  diff_1Y_RandFixed = diff_1Y_RandFixed(1:77386,1); clear temp
temp = sort(diff_1Y_FixedRand);  diff_1Y_FixedRand = diff_1Y_FixedRand(1:77386,1); clear temp
diff_1Y = [diff_1Y_RandRand diff_1Y_RandFixed diff_1Y_FixedRand];

temp = sort(diff_5Y_RandRand);  diff_5Y_RandRand = diff_5Y_RandRand(1:60847,1); clear temp
temp = sort(diff_5Y_RandFixed);  diff_5Y_RandFixed = diff_5Y_RandFixed(1:60847,1); clear temp
temp = sort(diff_5Y_FixedRand);  diff_5Y_FixedRand = diff_5Y_FixedRand(1:60847,1); clear temp
diff_5Y = [diff_5Y_RandRand diff_5Y_RandFixed diff_5Y_FixedRand];


%% Report and Save the Results
mean_diff_1Q = nanmean(diff_1Q,1); 
sd_diff_1Q   = nanstd(diff_1Q,1);
temp = sort(diff_1Q);
a = temp(:,1); b = temp(:,2); c = temp(:,3); % 8 seeds
qut_diff_1Q  = [quantile(a,[0.95])' quantile(b,[0.95])' quantile(c,[0.95])']
prob_1Q = [size(find(diff_1Q(:,1)<0),1) size(find(diff_1Q(:,2)<0),1) size(find(diff_1Q(:,3)<0),1)]./size(diff_1Q,1);

mean_diff_1Y = nanmean(diff_1Y,1);
sd_diff_1Y   = nanstd(diff_1Y,1);
temp = sort(diff_1Y);
a = temp(:,1); b = temp(:,2); c = temp(:,3); % 8 seeds
qut_diff_1Y  = [quantile(a,[0.95])' quantile(b,[0.95])' quantile(c,[0.95])'];
prob_1Y = [size(find(diff_1Y(:,1)<0),1) size(find(diff_1Y(:,2)<0),1) size(find(diff_1Y(:,3)<0),1)]./size(diff_1Y,1);

mean_diff_5Y = nanmean(diff_5Y,1);
sd_diff_5Y   = nanstd(diff_5Y,1);
temp = sort(diff_5Y);
a = temp(1:46486,1); b = temp(1:46486,2); c = temp(:,3); % 8 seeds
qut_diff_5Y  = [quantile(a,[0.95])' quantile(b,[0.95])' quantile(c,[0.95])'];
prob_5Y = [size(find(diff_5Y(:,1)<0),1) size(find(diff_5Y(:,2)<0),1) size(find(diff_5Y(:,3)<0),1)]./size(diff_1Y,1);

%% Report the results in Table 3
[qut_diff_1Y; qut_diff_5Y]

%% Save the results
fname = [cd, '\Results\simulation\CLM_LRR_SSY_NSIM',num2str(NSIM),'_T',num2str(T),'_SET',num2str(SET)];
save(fname,'diff_1Q','diff_1Y','diff_5Y','simmean_1Q','simmean_1Y','simmean_5Y','simsd_1Q','simsd_1Y','simsd_5Y');


