function out=kzspan(R1,R2,mu,nlags,fid)
% function out=kzspan(R1,R2,mu,nlags,fid)
%
% Kan-Zhou Analysis to calculate Spanning Tests Analytically
% Based on the paper by R. Kan and G. Zhou, "Tests of Mean-Variance Spanning"
% (See also hjshift)
% Test to see if R1 spans [R1 R2]
%
% R1, R2    datasets of gross returns 
% mu        2-vector with values restr the mean SDF.  Can pass []
% nlags     # of lags to use in standard error calcs
% fid       tells where to print results: 			[1]
%		0=none, 1/blank=screen, higher = file (use fopen)
%
% NOTE: No Guarantees with this program.   In particular, the J2 results
% look suspicious.  If anyone finds an error, I would appreciate hearing
% about it.
  
% WRITTEN BY: Mike Cliff,  Purdue Finance,  mcliff@mgmt.purdue.edu
% UPDATED   : 2/20/01

%=====================================================================
%   INITIALIZATIONS 
%=====================================================================
if nargin == 4
  fid = 1;
end
  
R = [R1 R2];
[T,k] = size(R1);            % k Basis Assets
n = cols(R2);                % n Test Assets
if isempty(mu)
  c1 = 0;                    c2 = 1;
else
  c1= mu(1);                 c2 = mu(2);
end
% --- Calculate Sample Moments ---------------------------------------
iotak = ones(k,1);                        iota = ones(n+k,1);
rbar = mean(R)';                          rbar1 = mean(R1)';
V = (R'*(R-repmat(rbar',T,1))/T);         Vi = V\eye(n+k);
V1 = (R1'*(R1-repmat(rbar1',T,1))/T);     V1i = V1\eye(k);
X = [ones(T,1) R1];
df = 2*n;

%=====================================================================
%   CONSTRUCT VARIOUS SPANNING TESTS
%=====================================================================

% --- Regression Approach --------------------------------------------
A = [1 zeros(k,1)'; 0 -ones(k,1)'];
B = (X'*X)\(X'*R2);
C = [zeros(n,1)'; -ones(n,1)'];
Theta = A*B-C;
e = R2 - X*B;
Sig = e'*e/T;
G = A*((X'*X/T)\eye(k+1))*A';
H = Theta*((Sig)\eye(n))*Theta';
ev = eig(H*((G)\eye(cols(G))));
LR = T*sum(log(1+ev));
W = T*sum(ev);
LM = T*sum(ev./(1+ev));

% --- Wa, Allows for Heteroskedasticity ------------------------------
a1 = rbar1'*V1i*rbar1;
b1 = rbar1'*V1i*iotak;
AT = [1+a1 -rbar1'*V1i; b1 -iotak'*V1i];
[junk,ST] = nwse(e,X,nlags);
vTheta = vec(Theta');
covterm = (kron(AT,eye(n))*ST*kron(AT',eye(n)))\eye(2*n);
Wa = T*vTheta'*covterm*vTheta;

% --- SDF Approach (Bekaert-Urias) -----------------------------------
b1 = Vi*(iota - c1*(1+rbar));
b2 = Vi*(iota - c2*(1+rbar));
b = [b1; b2];
C = kron(eye(2),V);
Ci = C\eye(cols(C));
F = [b1*rbar'  b2*rbar'];
Q = [zeros(n,k) eye(n)];
Q2 = kron(eye(2),Q);
sdf1 = c1 + (R-repmat(rbar',T,1))*b1;
sdf2 = c2 + (R-repmat(rbar',T,1))*b2;
e = [R.*repmat(sdf1,1,n+k)-1*(1-c1)];
e = [e R.*repmat(sdf2,1,n+k)-1*(1-c2)];
e = [e (R-repmat(rbar',T,1))];
[junk,S] = nwse(e,ones(T,1),nlags);
S11 = S(1:2*(n+k),1:2*(n+k));
S12 = S(1:2*(n+k),2*(n+k)+1:end);     
S21 = S(2*(n+k)+1:end,1:2*(n+k));
S22 = S(2*(n+k)+1:end,2*(n+k)+1:end);     
term = Q2*b;
term2 = S11;
covterm = (Q2*Ci*term2*Ci*Q2')\eye(2*n);
J2 = T*term'*covterm*term;
term2 = S11 - F'*S21 - S12*F + F'*S22*F;
covterm = (Q2*Ci*term2*Ci*Q2')\eye(2*n);
J1 = T*term'*covterm*term;

% --- Another SDF Version (DeSantis) ---------------------------------
%Ui = ((1+R)'*(1+R)/T)\eye(n+k);
Ui = (R'*R/T)\eye(n+k);
c1 = -(rbar'*Vi*iota + iota'*Vi*iota);
c2 = 1 + rbar'*Vi*rbar + rbar'*Vi*iota;
g1 = Ui*(iota - c1*(iota+rbar));
g2 = Ui*(iota - c2*(iota+rbar));
g = [g1; g2];
%sdf1 = c1 + (1+R)*g1;
%sdf2 = c1 + (1+R)*g2;
sdf1 = c1 + R*g1;
sdf2 = c1 + R*g2;
%e = [(1+R).*repmat(sdf1,1,n+k) (1+R).*repmat(sdf2,1,n+k)] - 1;
e = [R.*repmat(sdf1,1,n+k) R.*repmat(sdf2,1,n+k)] - 1;
[junk,S] = nwse(e,ones(T,1),nlags);
term = Q2*g;
term2 = kron(eye(2),Q*Ui)*S*kron(eye(2),Ui*Q');
J3 = T*term'*(term2\eye(cols(term2)))*term;


%=====================================================================
%   WRAP-UP STUFF
%=====================================================================

% --- Print the Results ----------------------------------------------
in.rnames = strvcat('LM','LR','W','Wa','J1','J2','J3');
in.cnames = strvcat('Stat','p-value');
out = [LM;LR;W;Wa;J1;J2;J3];
out = [out 1-chi2cdf(out,df)];
mprint1(out,in,fid);
