% Efficient portfolio bounds [Ferson and Siegel (2001)]
%
% INPUT   Em       : given unconditional mean of SDFs
%                  - scalar or vector
%         mu_R     : conditional mean of assets returns  
%                  - N*T matrix
%                  - N: number of securities
%                  - T: number of observations
%         sig2_R   : conditional covariance matrix of assets returns 
%         min_Pret : expected portfolio returns
%         target   : the location of an abitrary portfolio expected returns 
% OUTPUT  sig2_m : unconditional variance of SDFs given Em
%
function sig2_m = EFFICIENT_FS(Em,mu_R,sig2_R,mu_Pret,target,Returns)

[N,nobs] = size(mu_R);
e = ones(N,1);
% mu_Pret =-1.5:0.0005:1.5;     % setting expected portfolio returns 
lambda = zeros(N,N,nobs-1);
a1 = zeros(nobs-1,1);
a2 = zeros(nobs-1,1);
a3 = zeros(nobs-1,1);
for ii =1:nobs-1
    lambda(:,:,ii) = inv(mu_R(:,ii)*mu_R(:,ii)'+sig2_R);
    a1(ii,1) = 1/(e'*lambda(:,:,ii)*e);
    a2(ii,1) = e'*lambda(:,:,ii)*mu_R(:,ii)/(e'*lambda(:,:,ii)*e);
    a3(ii,1) = mu_R(:,ii)'*(lambda(:,:,ii)-(lambda(:,:,ii)*e)*(e'*lambda(:,:,ii))/(e'*lambda(:,:,ii)*e))*mu_R(:,ii);
end
alpha_1 = mean(a1,1);
alpha_2 = mean(a2,1);
alpha_3 = mean(a3,1);

sig2_Pret = NaN(length(mu_Pret),1);
for jj=1:size(mu_Pret,2)
    sig2_Pret(jj,1) = (alpha_1+alpha_2^2/alpha_3)-(2*alpha_2/alpha_3)*mu_Pret(jj)+((1-alpha_3)/alpha_3)*mu_Pret(jj)^2;
end
u_star =alpha_2/(1-alpha_3);               % global minimum variance portfolio
sig2_star = alpha_1-alpha_2^2/(1-alpha_3);
u_target = mu_Pret(target);                % an arbitrage expected target rate and corresponding variance
sig2_target = (alpha_1+alpha_2^2/alpha_3)-(2*alpha_2/alpha_3)*u_target+((1-alpha_3)/alpha_3)*u_target^2;
sig2_P = [sig2_star  0
         0      sig2_target];
mu_P = [u_star;u_target];
e1 = ones(2,1);

sig2_m = NaN(1,length(Em));
SDF_Z  = zeros(nobs,length(Em));

if nargin==6 % construct PtfReturns as x(Z)' * Returns
    x_Z = zeros(nobs,N,2); PtfReturns = NaN(nobs,2);
    for pp =1:2
        for ii =1:nobs-1
            x_Z(ii,:,pp)     = (e' * lambda(:,:,ii))/ (e' * lambda(:,:,ii) * e) + ((mu_P(pp) - alpha_2)/alpha_3) * mu_R(:,ii)' * ...
                (lambda(:,:,ii) - ((lambda(:,:,ii)*e)*(e'*lambda(:,:,ii)))/(e' * lambda(:,:,ii) * e)); % see eq. (11)
            
            PtfReturns(ii,pp) = x_Z(ii,:,pp) * Returns(ii+1,:)';% observed gross returns on the portfolios
        end
    end
end

for ii =1:size(Em)
    if nargin==6
            SDF_Z(:,ii) = (Em(ii) + (e1-Em(ii)*mu_P)'*inv(sig2_P)*(PtfReturns' - repmat(mu_P,1,nobs)))';
    end
    sig2_m(1,ii) = (e1-Em(ii)*mu_P)'*inv(sig2_P)*(e1-Em(ii)*mu_P);
end