% Optimally Scaled Hansen-Jagannathan bounds [Bekaert and Liu (2004)]
% -------------------------------------------------------------------------
% Proposition 1 in Bekaert and Liu (2004):
% If conditional moments of returns are known, then 
% sig2_OSB = sig2_GHT         [OSB: Optimally Scaled Variance Bound]
%                             [GHT: Gallant Hansen and Tauchen Bound]
% sig2_GHT =  E[(p_t-w*mu_t)'*(mu_t*mu_t'+SIG_t)^(-1)*(p_t-w*mu_t)]
%           - E[(p_t-w*mu_t)'*(mu_t*mu_t'+SIG_t)^(-1)*mu_t] 
%           [for GHT bounds we call our function OPTIMAL_GHT.m]
% sig2_OSB =  (a*(1-d)+b^2-2b*v+d*v^2)/(1-d)
% Notations: 
% a = E[p_t' *(mu_t*mu_t'+SIG_t)^(-1)*p_t];
% b = E[mu_t'*(mu_t*mu_t'+SIG_t)^(-1)*p_t];
% d = E[mu_t'*(mu_t*mu_t'+SIG_t)^(-1)*mu_t];
% w = (v-b)/(1-d)
% v = E[m_t]
% -------------------------------------------------------------------------
% INPUT   Em    :  given unconditional mean of SDFs
%                  - scalar or vector
%         mu   :  conditional mean of assets returns
%                  - T*N matrix
%                  - N: number of securities
%                  - T: number of observations
%         sig2   : conditional covariance matrix of assets returns
% OUTPUT  sig2_m : unconditional variance of SDFs given Em

function sig2_m = OSB_BL(v,mu,sig2_R)

[T,N] = size(mu);
p     = ones(T,N);     % prices, when the payoffs are gross returns
                       % p equal to 1
if size(sig2_R,3) == 1
    sig2 = repmat(sig2_R, [1 1 T]);
else
    sig2 = sig2_R;
end
temp_a = NaN(T,1); temp_b = NaN(T,1); temp_d = NaN(T,1);
for t = 1 : T
    temp_a (t,1)= p(t,:) *inv(mu(t,:)'*mu(t,:)+sig2(:,:,t))*p(t,:)';
    temp_b (t,1)= mu(t,:)*inv(mu(t,:)'*mu(t,:)+sig2(:,:,t))*p(t,:)';
    temp_d (t,1)= mu(t,:)*inv(mu(t,:)'*mu(t,:)+sig2(:,:,t))*mu(t,:)';
end
a = mean(temp_a);
b = mean(temp_b);
d = mean(temp_d);
clear temp_a temp_b temp_d

sig2_m = (a*(1-d)+b^2-2*b*v+d*v.^2)/(1-d);
              