% -------------------------------------------------------------------------
% Optimally Scaled Hansen-Jagannathan bounds [Bekaert and Liu (2004)]
% -------------------------------------------------------------------------
% See equation (33) in Bekaert and Liu (2004)
%
% Notations: 
% p_t  = 1; normolized price
% r_t  : is the vector of gross asset returns
% zstar_t  = (mu_t*mu_t'+SIG_t)^(-1)*(p_t-w*mu_t), optimal solution to the maximizition problem  
% v = E[m_t]
% w = (v-b)/(1-d)
% b = E[mu_t'*(mu_t*mu_t'+SIG_t)^(-1)*p_t];
% d = E[mu_t'*(mu_t*mu_t'+SIG_t)^(-1)*mu_t];
% 
% create a new matrix RR such that
% RR= [R sum(zstar_t'.R,2)]
% Then construt the original unconditional HJ bounds based on new gross
% returns matrix RR. 
% -------------------------------------------------------------------------
% INPUT   v     :  given unconditional mean of SDFs
%                  - scalar or vector
%         R     :  assets gross returns
%                  - T*N matrix
%                  - N: number of securities
%                  - T: number of observations
%         mu    :  conditional mean of assets returns
%                  - T*N matrix
%                  - N: number of securities
%                  - T: number of observations
%         sig2   : conditional covariance matrix of assets returns
% OUTPUT  sig2_m : unconditional variance of SDFs given Em

function sig2_m = OSB_BL_stacked(v,R,mu,sig2_R)

[T,N] = size(mu);
p     = ones(T,N);     % prices, when the payoffs are gross returns
                       % p equal to 1
if size(sig2_R,3) == 1
    sig2 = repmat(sig2_R, [1 1 T]);
else
    sig2 = sig2_R;
end
temp_a = NaN(T,1); temp_b = NaN(T,1); temp_d = NaN(T,1);
for t = 1 : T
    temp_a (t,1)= p(t,:) *inv(mu(t,:)'* mu(t,:) + sig2(:,:,t)) * p(t,:)';
    temp_b (t,1)= mu(t,:)*inv(mu(t,:)'* mu(t,:) + sig2(:,:,t)) * p(t,:)';
    temp_d (t,1)= mu(t,:)*inv(mu(t,:)'* mu(t,:) + sig2(:,:,t)) * mu(t,:)';
end

a = mean(temp_a);
b = mean(temp_b);
d = mean(temp_d);
    
sig2_m = NaN(1,length(v));
for ii =1:length(v) %% number of points on HJ bound
    vv = v(ii);
    w  = (vv-b)/(1-d);
    
    zt_star = NaN(N,T);
    for t = 1 : T
        zt_star(:,t) = inv(mu(t,:)'* mu(t,:) + sig2(:,:,t))*(p(t,:)-w*mu(t,:))';
    end
    
    RR      = [R sum(zt_star'.*R,2)];
    zJ      = zt_star'*ones(N,1);
    
    Ex     = mean(RR,1)';      %% mean gross returns
    cov_xx = cov(RR);          %% variance matrix
    
    bb = inv(cov_xx)*([ones(N,1); mean(zJ)]-vv*Ex);
    sig2_m(ii)=bb'*cov_xx*bb;  %% Volatility of SDF
end