function [ sdx, ser ] = gmm_sdev( x, lags, typeS )

% function [sd,ser] = sdev_gmm(x,lag,typeS )
% Computes the standard deviation of a time series x and its gmm-corrected
% standard error.
%
% INPUTS:
% x     : vectors of size "T" with the time series.
% lags  : number of lags used to estimate the asymptotic covariance matrix
%         to compute the standard error.
% typeS : Method used to estimate the asymptotic covariance matrix; =0
%         for even weighting; =1 for Newey-West weighting; =-1 for 
%         skipping the calculation of the standard error.
%
% OUTPUT:
% sd  : estimated standard deviations of x
% ser : gmm-corrected standard error of the standard deviations
%
% NOTE: In most cases it is convenient to estimate the asymptotic
% covariance matrix using the Newey-West estimator, so we should set
% typeS=1. The reason for this is that the simple even weighting could
% produce a non-positive definite covariance matrix.
%
% Written by Constantino Hevia, January 14, 2009.

% Perform checks
if size(x,1)<size(x,2); x=x'; end
T = size(x,1);
if lags<0
    error('ratsdev_gmm: lags must be nonnegative')
end
sdx = std(x);

% Compute standard errors
switch typeS
    case -1         % Do not compute standard error
        ser = [];
        return;
    case 0          % Use even weighting
        weight = 0;
    case 1          % Use Newey-West weighting
        weight = 1;
    otherwise
        error('correl_gmm: wrong typeS input')
end

xx = x.^2; 
mux = mean(x);
muxx = mean(xx); 

% Errors of moment conditions:
z = [ (x-mux)  (xx-muxx) ];

% Derivative of rsd(mu)
dsd = [ -mux/sdx
          0.5/sdx ];
          
% Efficient covariance matrix
Shat = z'*z/T;
for j=1:lags
    Shatadd = z(1:T-j,:)'*z(1+j:T,:)/T;
    Shat = Shat + (1-weight*j/(lags+1))*( Shatadd + Shatadd' );
end
ser = sqrt(dsd'*Shat*dsd/T);
