%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% Simlulation of Long-Run Risk Models
% 
% March, 2018  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. We simulate the LLR model by using both estimated and calibrated
%    parameters
% 2. The estimated parameters are taken from Bansal, Kiku and Yaron (2016, JME), 
%    RISKS FOR THE LONG RUN: ESTIMATION WITH TIME AGGREGATION
% REMARK: Set estimate = 0, the model is simulated by using calibrated parameters. 
%         Set estimate = 1, the model is simulated with estimated parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;

addpath('.\Library','.\Library\Library_LRR','.\data','.\Library.\Library_ConfidenceBands','.\Results\HJBounds')


                
burnin = 10000;

% step 1
NSIM = 10;
T    = 600000;
seed = 17234;
rand('seed', seed)
randn('seed', seed)

global mu rho vu sigma sigmaw phie gamma psi delta corr_dc
global phi pi_d phid mud A0 A2 zbar k1 beta_ae stochvol

stochvol=1; % 1: stochastic volatility
                    % 0: constant volatility
            
% The model parameters are based on the process given in equation

% The parameters are given in monthly terms

%% Parameter Setting

%% estimated parameters and SE (Bansal (2016, JME table 2))
%preference
gamma=9.67;    sig_gamma = 1.44;
psi  =2.18;    sig_psi   = 0.21;
delta=0.9990;  sig_delta = 0.0001;

freq_LR = 11;  sig_freq_LR = 2.16;  %decision periods per year
%mean consumption growth
mu = 0.0016;   sig_mu = 0.0005;
%mean dividend growth
mud= 0.0027;   sig_mud = 0.0010;

theta= (1-gamma)/(1-1/psi);

%Persistence
rho=0.9762;    sig_rho = 0.0035;
%DIV VOL multiple
phid=4.65;     sig_phid = 0.48;
%VOL multiple
phie=0.0318;   sig_phie = 0.0053;
%Leverage
phi=4.51;      sig_phi = 0.45;
%Vol Persistence
vu=0.9984;     sig_vu = 0.0007;
%Vol of Vol
sigmaw=2.12*10^(-6); sig_sigmaw = 5.32*10^(-7);
%Base SD
sigma=0.0070;  sig_sigma = 0.0009;
%
corr_dc = 0.51;sig_corr_dc = 0.10;
%Consumption Exposure
pi_d= 0.0;

totcensoring=0;

%% Model Simulation
% Em = 0.2:0.001:2;   % range of the mean of SDFs (consistenct with our bootstrap range)
horizon     = [3 12 60]; % 1-q, 1-y and 5-year holding

shock       = NaN(burnin+T,4,NSIM); 

%% Solve for price consumption ratio
%      zbar = fixed_point_pricecons(p0, N);
tol = 1e-05; maxit=100;
p0=3; % initial value
[k,zbar,err,P] = fixpt('pricecons',p0,tol,maxit);

k1 = exp(zbar)/(1+exp(zbar)); A1 = (1-1/psi)/(1-k1*rho);
k0 = log(1.0 + exp(zbar))-k1*zbar;
beta_ae = k1*A1*phie;
if stochvol==1
    A2 = 0.5*((theta - theta/psi)^2+(theta*beta_ae)^2)/(theta*(1-k1*vu));
else
    A2 = 0;
end
beta_aw = k1*A2;

A0 = (log(delta) + mu*(1-1/psi) + k0 +  beta_aw * sigma^2 * (1-vu) + ...
    0.5*theta* (beta_aw*sigmaw)^2) /(1-k1);
%      plotyy(1:516,xt,1:516,pc)

%% Solve for price dividend ratio
[k,zpdbar,err,P] = fixpt('pricediv',p0,tol,maxit); clear p0

k1m = exp(zpdbar)/(1+exp(zpdbar)); k0m = log(1.0 + exp(zpdbar)) - k1m*zpdbar;

A1m = (phi-1/psi)/(1-k1m*rho);
beta_me = k1m*A1m*phie;

lambda_e =  (1-theta)*beta_ae;
lambda_w =  (1-theta)*A2*k1;
lambda_n = -(- theta/psi + theta - 1); %== gamma
if stochvol==1
    A2m = ((1-theta)*A2*(1-k1*vu)+0.5*((pi_d - lambda_n)^2+(beta_me-lambda_e)^2 + phid^2))/((1-k1m*vu));
else
    A2m = 0;
end
beta_mw = k1m*A2m;

A0m = (theta*log(delta) + mu*theta*(1-1/psi) - lambda_w*(sigma^2)*(1-vu) + ...
    (theta-1)*( k0 + A0*(k1-1) ) - mu + ...
    k0m +  beta_mw * sigma^2 * (1-vu) + ...
    + mud + 0.5*(beta_mw-lambda_w)^2*(sigmaw)^2) /(1-k1m);

%% Simulation Results
for sim=1:NSIM
    fprintf('Iteration \t\t: %d \n',sim);
    
    %% Generate 4 sets of i.i.d. N(0,1)
    zeta = zeros(burnin+T,3);
    e  =randn(burnin+T,4);   % keep the same shocks
    
    gt1=zeros(burnin+T,1);
    xt =zeros(burnin+T,1);
    vol2=repmat(sigma^2,burnin+T,1);
    gdt1=zeros(burnin+T,1);
    
    for i=1:burnin+T-1
        if stochvol ==1
            vol2(i+1,1) = sigma^2 + vu*(vol2(i,1) - sigma^2 )+ sigmaw*e(i+1,1);
        else
            vol2(i+1,1) = sigma^2;
        end
        if vol2(i+1,1) < 0
            vol2(i+1)=0.0001*sigma^2;
            totcensoring=totcensoring+1;
        end
        gt1(i+1,1) =mu  +     xt(i,1)  +        sqrt(vol2(i,1)) * e(i+1,2);
        gdt1(i+1,1)=mud + phi*xt(i,1)  + phid * sqrt(vol2(i,1)) * e(i+1,3) + pi_d * sqrt(vol2(i,1)) * e(i+1,2); %In the BY model \pi_d = 0 so BY solutions are a special case.
        xt(i+1,1)  =      rho*xt(i,1)  + phie * sqrt(vol2(i,1)) * e(i+1,4);
        
        zeta(i+1,1) = sqrt(vol2(i,1)) * e(i+1,2);
        zeta(i+1,2) = sqrt(vol2(i,1)) * e(i+1,4);
        zeta(i+1,3) = sigmaw*e(i+1,1);
    end
    xt  = trimr(xt,burnin,0) ; vol2 = trimr(vol2,burnin,0);
    gt1 = trimr(gt1,burnin,0); gdt1 = trimr(gdt1,burnin,0);
    
    zeta = trimr(zeta,burnin,0);
    % plot([gt1 xt])
    
    log_pc_ratio=A0 +A1 *xt+A2 *vol2;
    log_pd_ratio=A0m+A1m*xt+A2m*vol2;
    
    rretMkt  = k0m +  k1m*log_pd_ratio(2:end) + gdt1(2:end,1)- log_pd_ratio(1:end-1);
    rretCons = k0  +  k1 *log_pc_ratio(2:end) + gt1(2:end,1) - log_pc_ratio(1:end-1);
    
    %Consumption claim
    var_ra  =(beta_ae^2 + 1)*vol2(1:end-1,1) +(beta_aw*sigmaw)^2; %Annualized Market Volatility sqrt(mean(var_rm)*12)*100
    ConsumptionRP=lambda_n * 1 * vol2(1:end-1,1) + beta_ae*lambda_e    *vol2(1:end-1,1) + beta_aw*lambda_w*sigmaw^2 - 0.5*var_ra;
    %Dividend claim
    var_rm  =(beta_me^2 + phid^2)*vol2(1:end-1,1) +(beta_mw*sigmaw)^2; %Annualized Market Volatility sqrt(mean(var_rm)*12)*100
    MarketRP     =lambda_n * pi_d * vol2(1:end-1,1) + beta_me*lambda_e *vol2(1:end-1,1) + beta_mw*lambda_w*sigmaw^2 - 0.5*var_rm;
    %% BY (2004) risk-free rate
    Et_gt1   = mu + xt(1:end-1,1);
    vart_mt1 = (lambda_n^2+lambda_e^2)*vol2(1:end-1,1)+lambda_w^2*sigmaw^2;
    rrf_by   = -log(delta) + (1/psi)*Et_gt1 + (1-theta)/theta*ConsumptionRP - (1/(2*theta))*vart_mt1;
    %% Gallant risk-free rate
    Gamma0_1 = log(delta) - (1/psi)*mu;
    Gamma0_2 = -0.5*theta*(theta - 1)*(k1*A2*sigmaw)^2;
    Gamma0 = Gamma0_1 + Gamma0_2;
    Gamma1 = -1/psi;
    Gamma2 = (theta - 1)*(k1*vu - 1)*A2;
    Y = zeros(size(rretMkt,1),3);
    Y(:,1) = ones(size(rretMkt,1),1);
    Y(:,2) = xt(1:end-1,1);
    Y(:,3) = vol2(1:end-1,1);
    rrf_g = -Gamma0-Gamma1*Y(:,2)-Gamma2*Y(:,3)-0.5*lambda_n^2*vol2(1:end-1,1)- 0.5*lambda_e^2*vol2(1:end-1,1)-0.5*lambda_w^2*sigmaw^2;
    
    %% Pricing Kernel (IMRS)
    log_mrs  = theta*log(delta)-(theta/psi)*gt1(2:end,1)+(theta-1)*rretCons;
    %     log_mrs_gallant = Gamma0 + Gamma1*Y(:,2) + Gamma2*Y(:,3) - lambda_n*zeta(2:end,1) - lambda_e*zeta(2:end,2) - lambda_w*zeta(2:end,3);
    MRS      = exp(log_mrs);
    sim_SDF(:,sim) = MRS;
    
    %% Construct 1-Quarter and 5-year MRS from monthly data
    % 1-Quarter
    MRS_1Q = exp(chgfreq(log_mrs,horizon(1),horizon(1),horizon(1)-1));
    sim_SDF_1Q   (:,sim) = MRS_1Q;
    simmean_1Q(sim,1)   = mean(MRS_1Q,1);
    simsd_1Q(sim,1)     = std(MRS_1Q,1);
  
    
    % 1-Year
    MRS_1Y = exp(chgfreq(log_mrs,horizon(2),horizon(2),horizon(2)-1));
    sim_SDF_1Y   (:,sim) = MRS_1Y;
    simmean_1Y(sim,1)   = mean(MRS_1Y,1);
    simsd_1Y(sim,1)     = std(MRS_1Y,1);
    
    % 5-Year
    MRS_5Y = exp(chgfreq(log_mrs,horizon(3),horizon(3),horizon(3)-1));
    sim_SDF_5Y   (:,sim) = MRS_5Y;
    simmean_5Y(sim,1)   = mean(MRS_5Y,1);
    simsd_5Y(sim,1)     = std(MRS_5Y,1);
  end

simmean_c = [simmean_1Q(:,1) simmean_1Y(:,1) simmean_5Y(:,1)];
simsd_c   = [simsd_1Q(:,1)   simsd_1Y(:,1)   simsd_5Y(:,1)];

SET = 5;
SDF_MH=[cd,'\Results\HJBounds\SDF_MH_SET',num2str(SET)];
load(SDF_MH)

figure
for h =1:3
    hold off
    subplot(1,3,h); 
    plot(E_m, sqrt(sig2_BL(h,:)),'k','LineWidth',4); hold on
    plot(E_m, sqrt(sig2_HJ(h,:)),'Color',[0.48 0.06 0.89],'LineStyle','--','LineWidth',4.5); hold on
    if h == 1
        xlim([0.98 1.01]); title('1-Quarter')
        ylim([0 1.6])
    elseif h ==2
        xlim([0.8 1.025]); title('1-Year')
         ylim([0 2.5])
    elseif h ==3
        xlim([0.3 1.28]); title('5-Year')
         ylim([0 6.5])
    end
    x0 = mean(simmean_c([1:5 7:10],h)); y0 = mean(simsd_c([1:5 7:10],h)); % Center of ellip

    plot(x0,y0,'^r','MarkerSize',12,'MarkerFaceColor','r'); hold on
    xlabel('E(m)')
    ylabel('standard deviation')
end
% h_legend = legend('predictors-based bound','HJ bound','simulated value - Baseline','simulated value - Permanent and one period','1-sd confidence area');
h_legend = legend('Orientation','horizontal','predictors-based bound','HJ bound');
set(h_legend,'FontSize',13);


%% Save the results
fname = [cd, '\Results\simulation\sim_LRRBKY2016_NSIM',num2str(NSIM),'_T',num2str(T)];
save(fname,'simmean_c', 'simsd_c');



