%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% Simlulation of Long-Run Risk Models by Schorfheide, Song and Yaron(2018)
% 
% June, 2018   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. We simulate the LLR-SSY (Schorfheide, Song and Yaron, 2018) model by using estimated parameters
% 2. calculate the estimated standard deviation of mu_SDF (1Q, 1Y and 5Y) and std_SDF
% 3. 10 simulations with 600,000 months
%
% Remark: we adopte the median of estimated parameter values reported in Table 5 to simulate the model. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;

addpath('.\Library','.\Library\Library_LRR_SSY','.\data','.\Results\HJBounds','.\Results\simulation')
addpath('.\Library\gmm;','.\Results\HJBounds','.\Library','.\data','.\Library\minz;','.\Library\HJBLibrary','.\Library\Library_CHJ')

             
burnin = 240;

% step 1
NSIM = 10; 
T    = 600000;

%%

seed = 17234;
rand('seed', seed)
randn('seed', seed)

%%
global delta psi gamma rho rho_lambda sigma_lambda rho_hx rho_hc varphi_c varphi_x sigma sigma_hc sigma_hx mu_c

            
% The model parameters are based on the process given in equation

% The parameters are given in monthly terms

%% Model Parameter Setting

%% estimated parameters and SE

% The parameters are given in monthly terms
% We use the estimated posterior median vaule of all the estimated parameters from
% Schorfheide, Song and Yaron (2016, Table 5)

% Household Preferences
delta      = 0.999;     
psi         = 1.935;     sig_psi = (1.935-1.134)/1.64;         % we constructed based on the quantile values in Table 5
gamma  = 8.598;    sig_gamma = (8.598-5.441)/1.64;
% 5.441 (5% quantile); 12.969 (95% quantile); 8.598(50% quantile)

theta= (1-gamma)/(1-1/psi);

% Preference Risk
rho_lambda   = 0.9559;   sig_rho_lambda = (0.9559-0.9157)/1.64;
sigma_lambda = 0.0005;  sig_sigma_lambda = (0.0005-0.0003)/1.64;

% Consumption Growth Process
mu_c      = 0.0016; % fixed, see table 5
rho       = 0.9872;  sig_rho = (0.9872-0.9486)/1.64;
varphi_x  = 0.2315;  sig_varphi_x = (0.2315-0.1388)/1.64;
sigma     = 0.0032; sig_sigma = (0.0032-0.0020)/1.64;
rho_hc    = 0.9914;  sig_rho_hc = (0.9914-0.9733)/1.64;
varphi_c  = 1; % fixed in the paper, normalized based on the consumption series
sigma2_hc = 0.0088; sig_sigma2_hc = (0.0088-0.0074)/1.64;
sigma_hc  = sqrt(sigma2_hc); sig_sigma_hc = (sqrt(0.0088)-sqrt(0.0074))/1.64;
rho_hx    = 0.9943;  sig_rho_hx = (0.9943-0.9874)/1.64;
sigma2_hx = 0.0039; sig_sigma2_hx = (0.0039-0.0027)/1.64;
sigma_hx  = sqrt(sigma2_hx); sig_sigma_hx = (sqrt(0.0039)-sqrt(0.0027))/1.64;

% Dividend Growth Process
mu_d      = 0.0010; % fixed, see table 5
phi       = 4.147;   sig_phi = (4.147-2.816)/1.64;
pi        = 1.544;   sig_pi = (1.544-0.204)/1.64;
varphi_d  = 5.023;   sig_varphi_d = (5.023-3.562)/1.64;
rho_hd    = 0.9665;  sig_rho_hd = (0.9665-0.9475)/1.64;
sigma2_hd = 0.0393; sig_sigma2_hd = (0.0393-0.0174)/1.64;
sigma_hd  = sqrt(sigma2_hd); sig_sigma_hd = (sqrt(0.0393)-sqrt(0.0174))/1.64;

% Consumption Measurement Error
sigma_epsilon   = 0.0010;  sig_sigma_epsilon = (0.0010-0.0006)/1.64;
sigma_epsilon_a = 0.0231;  sig_sigma_epsilon_a = (0.0231-0.0061)/1.64; 

% measurement error standeard deviations, fixed, see table 5, 10% of the
% sample satandeard deviation of the dividend growth and the risk-free rate
% % % sigma_epsilon_da = 0.1 * ; 
% % % sigma_epsilon_f  = 0.1 * ;
          
%% MODEL SIMULATION  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Em = 0.2:0.001:2;   % range of the mean of SDFs (consistenct with our bootstrap range)
horizon     = [3 12 60]; % 1-q, 1-y and 5-year holding

for ss = 1:size(seed,2)  
    rand('seed', seed(ss))
    randn('seed', seed(ss))
 
for sim = 1:NSIM
   
    %% Solve for price consumption ratio 
    options = optimset('MaxIter', 1e20, 'MaxFunEvals', 1e20, 'TolFun',1e-20, 'Display', 'off');
    zcfcn   = @(x) z_bar(x);
    zc0     = 3;
    [zmean] = fsolve(zcfcn,zc0,options);
    zmean = 3; % 3 for median gamma and low gamma; 2.8 for high gamma 
   
    %% %%%%%%%%%%%%%%%%%%%%%%%%%
    
    k1 = exp(zmean)/(1+exp(zmean)); 
    if k1>0.9999
        k1=0.9999;
    end
    k0 = log(1.0 + exp(zmean))-k1*zmean;
    
    A1 = (1-1/psi)/(1-k1*rho);
    A1_lambda = rho_lambda/(1-k1*rho_lambda);
    
    A2_x = 0.5*theta*(k1*A1)^2/(1-k1*rho_hx);
    A2_c = 0.5*theta*(1-1/psi)^2/(1-k1*rho_hc);
    
    sigma_wc = 2*(varphi_c*sigma)^2*sigma_hc;
    sigma_wx = 2*(varphi_x*sigma)^2*sigma_hx;
    
    A0_1 = log(delta)+k0+mu_c*(1-1/psi)+k1*A2_x*(1-rho_hx)*(varphi_x*sigma)^2 + k1*A2_c*(1-rho_hc)*(varphi_c*sigma)^2;
    A0_2 = 0.5*theta*((k1*A1_lambda+1)^2*sigma_lambda^2+(k1*A2_x*sigma_wx)^2+(k1*A2_c*sigma_wc)^2);
    A0   = (A0_1+A0_2)/(1-k1); 
        
    %% Simulation Results
    fprintf('Iteration \t\t: %d \n',sim);
    
    %% Generate 5 sets of i.i.d. N(0,1)
    eta = randn(burnin+T,3);
    w   = randn(burnin+T,2);
    
    %%
    gt1=zeros(burnin+T,1);
    xt =zeros(burnin+T,1);
    xt_lambda =zeros(burnin+T,1);
    sigma2_ct = zeros(burnin+T,1);  sigma2_ct(1) = sigma^2;
    sigma2_xt = zeros(burnin+T,1);  sigma2_xt(1) = (varphi_x*sigma)^2;
    log_pc_ratio = zeros(burnin+T,1);
    rretCons = zeros(burnin+T,1);
    
    %%
    h_x = zeros(burnin+T,1); 
    h_c = zeros(burnin+T,1);
    sigma_ct = zeros(burnin+T,1);  
    sigma_xt = zeros(burnin+T,1);  
    %%
    
    for i=1:burnin+T-1        
    
        h_c(i+1,1) = rho_hc * h_c(i,1) +sigma_hc*w(i+1,1);
        h_x(i+1,1) = rho_hx *  h_x(i,1) +sigma_hx*w(i+1,2);
        sigma_ct(i+1,1) = varphi_c*sigma*exp(h_c(i+1,1));   sigma2_ct(i+1,1) = sigma_ct(i+1,1)^2;
        sigma_xt(i+1,1) = varphi_x*sigma*exp(h_x(i+1,1));   sigma2_xt(i+1,1) = sigma_xt(i+1,1)^2;
        %% Alternative way to compute sigma2_ct and sigma2_xt (see Appendix C)             
%         sigma2_ct(i+1,1) = (1-rho_hc)*(varphi_c*sigma)^2 + rho_hc*sigma2_ct(i,1) + sigma_wc*w(i+1,1);
%         sigma2_xt(i+1,1) = (1-rho_hx)*(varphi_x*sigma)^2 + rho_hx*sigma2_ct(i,1)  + sigma_wx*w(i+1,2);
%         
%         if sigma2_ct(i+1,1)<0
%             sigma2_ct(i+1,1) =0.0001*sigma^2;
%         end
%         
%         if sigma2_xt(i+1,1)<0
%             sigma2_xt(i+1,1) =0.0001*(varphi_x*sigma)^2;
%         end
         %%
        
        xt(i+1,1)  = rho*xt(i,1)+sqrt(sigma2_xt(i,1)) * eta(i+1,1);
        gt1(i+1,1) = mu_c + xt(i,1) + sqrt(sigma2_ct(i,1)) * eta(i+1,2);
        
        xt_lambda(i+1,1)  = rho_lambda * xt_lambda(i,1) + sigma_lambda * eta(i+1,3);
        
        log_pc_ratio(i+1,1) = A0 + A1 *xt(i+1,1)+ A1_lambda *xt_lambda(i+1,1) + A2_c*sigma2_ct(i+1,1) + A2_x*sigma2_xt(i+1,1);
        rretCons(i+1,1) = k0 + k1*log_pc_ratio(i+1,1) - log_pc_ratio(i,1) + gt1(i+1,1);
    end
    sigma2_ct  = trimr(sigma2_ct,burnin,0) ;
    sigma2_xt  = trimr(sigma2_xt,burnin,0) ;
    xt  = trimr(xt,burnin,0) ; xt_lambda  = trimr(xt_lambda,burnin,0) ;
    gt1 = trimr(gt1,burnin,0); 
    log_pc_ratio = trimr(log_pc_ratio,burnin,0); 
    rretCons = trimr(rretCons,burnin,0);    
    
    %% Pricing Kernel (IMRS)
    log_mrs  = theta*log(delta) -(theta/psi)*gt1 + theta*xt_lambda  +(theta-1)*rretCons;
    MRS      = exp(log_mrs);
    
    %% Construct 1-Quarter and 5-year MRS from monthly data
    MRS_1Q = exp(chgfreq(log_mrs,horizon(1),horizon(1),horizon(1)-1));   
    MRS_1Y = exp(chgfreq(log_mrs,horizon(2),horizon(2),horizon(2)-1));  
    MRS_5Y = exp(chgfreq(log_mrs,horizon(3),horizon(3),horizon(3)-1)); 
    
    coult1 = 0; coult2 = 0; coult3 = 0;
    if T == 742
        if isnan(mean(MRS_1Q))
            coult1= count1;
        else
            sim_MRS_1Q(:,sim) = MRS_1Q;
            coult1= count1+1;
        end
        if isnan(mean(MRS_1Y))
            coult2= count2;
        else
            sim_MRS_1Y(:,sim) = MRS_1Y;
            coult2= count2+1;
        end
        if isnan(mean(MRS_5Y))
            coult3= count3;
        else
            sim_MRS_5Y(:,sim) = MRS_5Y;
            coult3= count3+1;
        end
    end
    
    % Compute the mean, mean[SDF]
    simmean_1Q(sim,ss)   = mean(MRS_1Q,1);   
    simmean_1Y(sim,ss)   = mean(MRS_1Y,1);    
    simmean_5Y(sim,ss)   = mean(MRS_5Y,1);

    simsd_1Q(sim,ss)     = std(MRS_1Q,1);
    simsd_1Y(sim,ss)     = std(MRS_1Y,1);
    simsd_5Y(sim,ss)     = std(MRS_5Y,1);  
end

end

%% Save the results
SET = 1;

if gamma == 5.441
    fname = [cd, '\Results\simulation\sim_LRRSSY_NSIM',num2str(NSIM),'_T',num2str(T),'_LowGamma'];
elseif gamma == 12.969
    fname = [cd, '\Results\simulation\sim_LRRSSY_NSIM',num2str(NSIM),'_T',num2str(T),'_HighGamma'];
elseif gamma == 8.598
 fname = [cd, '\Results\simulation\sim_LRRSSY_NSIM',num2str(NSIM),'_T',num2str(T),'_SET',num2str(SET)];
end
save(fname,'simmean_1Q','simmean_1Y','simmean_5Y','simsd_1Q','simsd_1Y','simsd_5Y');

%% Plot
SET = 1;        % 1: SET A: stock market + risk free bond + 5-year government bonds
                % 2: SET B: stock market + risk free bond + 5-,7-, 10-, 20-, 30-year government bonds
SDF_MH=[cd,'\Results\HJBounds\SDF_MH_SET',num2str(SET)];
load(SDF_MH)

meanMRS_LRR_SSY = [simmean_1Q simmean_1Y simmean_5Y];
sdMRS_LRR_SSY   = [simsd_1Q simsd_1Y simsd_5Y];

figure
for h =1:3
    hold off
    subplot(1,3,h); 
    plot(E_m, sqrt(sig2_BL(h,:)),'k','LineWidth',4); hold on
    plot(E_m, sqrt(sig2_HJ(h,:)),'Color',[0.48 0.06 0.89],'LineStyle','--','LineWidth',4.5); hold on
    if h == 1
        xlim([0.985 1.007]); title('1-Quarter')
        ylim([0 1.2])
    elseif h ==2
        xlim([0.93 1.035]); title('1-Year')
        ylim([0 2])
    elseif h ==3
        xlim([0.5 1.6]); title('5-Year')
        ylim([0 5])
    end
    plot(mean(meanMRS_LRR_SSY([2:10],h)),mean(sdMRS_LRR_SSY([2:10],h)),'^b','MarkerSize',17,'MarkerFaceColor','b');hold on
    xlabel('E(m)')
    ylabel('standard deviation')
end
% h_legend = legend('predictors-based bound','HJ bound','simulated value with estimated parameters','1-sd confidence area',...
%                  'simulated value with calibrated parameters');
h_legend = legend('predictors-based bound','HJ bound','simulated value SSY');
set(h_legend,'FontSize',13);