%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% Main code for replication of Nakamura, Steinsson, Barro adn Ursua(2013):
% Crises and Recoveries in an Empirical Model of Consumption Disasters
%
% November, 2017    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code replicates the main results of paper: Crises and Recoveries in an Empirical 
%     Model of Consumption Disasters.
%     We write this code based on the replication package by Nakamura, Steinsson, Barro 
%     and Ursua, DisastersDataAndPrograms.zip.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  README
% 1. Main code to replicate Rare Disaster model is: RepRareDisaster_NSBU.m.
%     * This code replicate specification 1, 4, 8 in Table 7, Figure 7
%     * Baseline in Table 7; Specification 2 & 3 & 9 are computed in RepRareDisaster_NSBUPerm.m
% 
% 2. We use the estimated parameters directly from their paper. Also we only focus on the case of US. 
% 
% 3. Solve PD ratio: 
%    1) [line 153-154] 
%       nextStateFuncGen.m: generate next states for both disaster and no disaster cases.
%       All the results are saved in folder 'Results-->replication-->RepNSBU
%       You only need to run it once to save the results, later on, you can recall the results directly. No need to run it everytime.
%    2) [line 155-161]   
%       PDivs_EZW.m: solve PD ratio for both equity and one period bond with Epstein-Zin preference.
%       Results are saved in folder 'Rsults-->replication-->RepNSBU
%       You only need to run it once to save the results, later on, you can recall the results directly. No need to run it everytime.
%
%       a) PDivIntFuncGenEq.m : sub-function for equity 
%       b) PDivIntFuncGenOnePerBond.m : sub-function for bond
% 
%
% 4. NoDisasters -- 0: Full Sample in Table 7
%                   1: No Disaster in Table 7
%
% N.B. : To solve for the PD ratio we use ``sigmaEps, post-1946'' and ``muPost, post-1973
%  see function ``createIntGrid(DistDropLongRun, DistDropPeakTrough)'',
% ``PDivIntFuncGenEq(currentState, intGrid,weightsOverIntGrid,nextPDivVecDis,nextPDivVecNoDis)'' and
% ``DistDisasters(nSimDistDis,maxDistDrop)''
%  THIS IS JUSTIFIED BY FOOTNOTE 31 IN THE PAPER
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
clear all; close all; clc; 
addpath('.\Library\Library_RareDisaster_NSBU','.\Results\HJBounds','.\Results\replication\RepNSBU','.\data','.\Library')
 

global nSimDistDis maxDistDrop muPost sigmaEps  sigmaEta  meanPhi  sigmaPhi  meanTheta...
          sigmaTheta sigmaNu rhoZZ  p_W p_CbW p_CbI p_Ce
      
global nStateGridEps nStateGridZZ ZZshrinkFactor fracCutoffZZ
global nIntGridEps nIntGridEta nIntGridTheta nIntGridPhi nIntGridNu
global nSdEps  nSdEta nSdTheta nSdPhi nSdNu  

global  rhoU  gammaU  psiU  thetaU onePerPermDis MultiPerPermDis NoShortRunShock onePerDis
global  payGrowth  pDefaultLong pDefaultShort debtEqRatio tolIntEq maxIter tolIntBond

% For getDistasterDist
NoDisasters    = 0;          %0: case with disasters, (Full Sample in Table 7)
                             %1: case without disasters, (No Disasters in Table 7)
                             
% type of Disaster [Set all equal to 0 to obtain the results in Baseline
% case], For permanent/permanent one-period, check code SimRareDisaster_NSBUPerm.m
onePerPermDis  = 0;          % Permanent and one period, specification 3 in Table 7
MultiPerPermDis= 0;          % Permanent, specification 2 in Table 7
onePerDis      = 0;          % one period, specification 10 in Table 7, we did not replicate this case
                                        
tolIntEq       = 0.000001;   %Toleraunce for iterations on Equity integral
tolIntBond     = 0.000001;   % Tolerance for iterations on Bond integral
tolIntPerp     = 0.000001;   % Tolerance for iterations on Perpetuity integral
tolIntScaEq    = 0.000001;   % Tolerance for iterations on "scaled" Equity integral
tolIntOPEq     = 0.000001;   % Tolerance for iterations on One Period Equity integral

maxIter        = 1500;         % Maximum number of iterations

% Utility parameters
gammaU     = 6.4;

%% Low Gamma v.s. High Gamma, case 5&6
% gammaU = 4.4;
% gammaU = 8.4;
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psiU           = 2;                 
rhoU           = 0.034;           
beta           = exp(-0.034);
thetaU         = (1-gammaU)/(1-1/psiU);

% Payoff properties of long term bond
payGrowth      = 0.90;      % Gross growth rate of payoff on the perpetuity
pDefaultLong   = 0.40;      % Probability that the perpetuity will suffer a partial default in each disaster period
pDefaultShort  = 0.00;      % Probability that the one period bond will suffer a partial default in each disaster period

% Leverage on leveraged equity
debtEqRatio    = 0.50;
initPDivEq     = log(12);

% For getDistasterDist
nSimDistDis    = 1000;
maxDistDrop    = 1.0;            % max percentage drop in PeakTrough and LongRun

% State Gridsize
nStateGridEps  = 1;
nStateGridZZ   = 601;
nStateGrid     =  nStateGridEps*nStateGridZZ*2;

% Integration Gridsize
nIntGridEps    = 1;
nIntGridEta    = 9;
nIntGridTheta  = 31;
nIntGridPhi    = 31;
nIntGridNu     = 1;
nIntGrid       = nIntGridEps*nIntGridEta*nIntGridTheta*nIntGridPhi*nIntGridNu;

% Grid limits in units of standard deviations
nSdEps         = 2;
nSdEta         = 3.5;
nSdTheta       = 3.5;
nSdPhi         = 3.5;
nSdNu          = 2;
fracCutoffZZ   = 0.025;

% ZZ Grid parameters
ZZshrinkFactor = 0.97;

% Parameters for simulating a "typical disaster"
nPrePeriods    = 5;
nDisPeriods    = 6;
nPostPeriods   = 25;

%% Get Parameter Estimates (we take the mean of posterior estimated parameters from paper directly, Table 1)

meanPhi        = -0.111;   sigmeanPhi    = 0.008;     % theta: a one-off permanent shift in the level of potential consumption, x_t

meanTheta      = -0.025;   sigmeanTheta  = 0.007;     % phi: a shock that causes a temporary drop in consumption due to the disaster at time t

p_W            = 0.037;    sigp_W        = 0.016;     % probability of a world-wide disaster 
p_CbW          = 0.623;    sigp_CbW      = 0.076;     % probability that a particular country will enter a disaster when a world disaster begins
p_CbI          = 0.006;    sigp_CbI      = 0.003;     % probability that a particular country will enter a disaster on its own?
p_Ce           = 1-0.835;                             % probability to exit the disaster
p_CeC          = 1-p_Ce;   sigp_CeC      = 0.027;

rhoZZ          = 0.5;      sigrho_ZZ     = 0.034;     % firts order autoregressive coefficient of z_t

sigmaTheta     = 0.121;    sigsigmaTheta = 0.015;
sigmaPhi   = 0.083;    sigsigmaPhi   = 0.006;     % theta: a one-off permanent shift in the level of potential consumption, x_t


% US country-specific estimates of parameters. See Table 3 and Table 4.
muPost         = 0.022;    sigmu         = 0.003;    % mean growth rate of potential consumption, post-1973 
sigmaEta       = 0.018;    sigsigmaEta   = 0.002;
sigmaEps       = 0.003;    sigsigmaEps   = 0.002;    % post-1946
sigmaNu        = 1e-5;        % this one is introduced to ad the convergence of the numberical algorithm
                              % set unimportant parameters to samll value, see in paper and their code

%% Generate Disasters and create grids
[DistDropBeginEnd,DistDropPeakTrough,DistDropLongRun,DistMaxZZ,DistMinZZ,DistXX,...
    DistZZ, DistCC, medianCC, q25CC, q75CC] =DistDisasters(nSimDistDis,maxDistDrop);


% Create State Grid
[stateGrid,  stateGridII, stateGridEps, stateGridZZ]   = createStateGrid(DistMinZZ,DistMaxZZ);

% Create Integration grid
[intGrid, intGridEps, intGridEta, intGridTheta, intGridPhi, intGridNu, tempGridEps, tempGridTheta, tempGridPhi, ...
    tempGridEta, tempGridNu ] = createIntGrid(DistDropLongRun, DistDropPeakTrough);

% Calculate weights over IntGrid
weightsOverIntGrid = calcWeights(intGridEps,  intGridEta, intGridTheta, intGridPhi, intGridNu,...
    tempGridEps, tempGridTheta, tempGridPhi, tempGridEta, tempGridNu,...
    DistDropLongRun, DistDropPeakTrough);

%% Solve PD ratio
% % % %  Generate nextStates for both disaster and no disaster cases
% % %  nextStateFuncGen (stateGrid, stateGridEps, stateGridZZ, intGrid, intGridEps, intGridEta, ...
% % %     intGridTheta, intGridPhi, intGridNu,0);
% % % % Solving PD ratio for both equity and one period bond(bill)
% % % 
% % % [PDivVecEq, PDivVecOnePerBond] = PDivs_EZW (stateGrid,intGrid, weightsOverIntGrid,initPDivEq);
% % % fname = [cd, '\Results\simulation\SimNSBU\logPDivRatio'];
% % % % fname = [cd, '\Results\simulation\SimNSBU\logPDivRatio_LowGamma'];
% % % % fname = [cd, '\Results\simulation\SimNSBU\logPDivRatio_HighGamma'];
% % % save(fname,'PDivVecEq','PDivVecOnePerBond')


load(logPDivRatio); %#check size(stateGrid,1) == size(PDivVecEq) == size(PDivVecOnePerBond)
% load(logPDivRatio_LowGamma);
% load(logPDivRatio_HighGamma);
%% Model simulation
TSIM =  600000/4;%;;600000/4;200000
NSIM =  1; %1;
burnin= 240;

rand('seed', 77645)
randn('seed', 77645)

if onePerPermDis == 1
    ThetaDist = DistDropPeakTrough;
end

for sim = 1:NSIM
    
    sim
    
    % Set initial values
    II          =  zeros(TSIM+burnin,1);
    IIWorld     =  zeros(TSIM+burnin,1);
    XX          =  zeros(TSIM+burnin,1);
    ZZ          =  zeros(TSIM+burnin,1);
    Eps         =  zeros(TSIM+burnin,1);
    CC          =  zeros(TSIM+burnin,1);
    Eta         =  zeros(TSIM+burnin,1);
    Phi         =  zeros(TSIM+burnin,1);
    Theta       =  zeros(TSIM+burnin,1);
    Nu          =  zeros(TSIM+burnin,1);
    
    XX(1)       = 1;
    CC(1)       = XX(1) + ZZ(1) + Eps(1);
    
    
    % Simulate model
    for tt = 2:TSIM+burnin
        IIWorld(tt)         = binornd(1,p_W);
        
        if  II(tt-1) == 1
            temppp = 1-p_Ce;      %ppC(2)
        else
            if IIWorld(tt) == 1
                temppp = p_CbW;  %ppC(3)
            else
                temppp = p_CbI;  %ppC(1)
            end
        end
        
        if NoDisasters ~=0
            II(tt)    = 0;
        else
            II(tt)    = binornd(1,temppp);
        end
        
        if II(tt) == 1
            if  onePerPermDis==0
                Theta(tt) = meanTheta+sigmaTheta*randn(1);
            elseif onePerPermDis == 1
                tempXX    = unifrnd(0,1);
                tempXX    = max(1,round(tempXX*length(ThetaDist)));
                Theta(tt) = ThetaDist(tempXX);
            end
        end
        
        if onePerPermDis == 0
            k = 1;
            while k < 2
                rdnumber = meanPhi + sigmaPhi*randn(1,1);
                if rdnumber < 0 & rdnumber> -1000
                    Phi(tt) = rdnumber; % truncated normal variable
                    k = k +1;
                else
                    k = k;
                end
            end
        else
            Phi(tt) = Theta(tt);
        end
              
        Eps(tt)   = sigmaEps*randn(1);
        Eta(tt)   = sigmaEta*randn(1);
        Nu(tt)    = sigmaNu *randn(1);
        
        XX(tt)    = XX(tt-1) + muPost + Eta(tt) + Theta(tt)*II(tt);
        ZZ(tt)    = rhoZZ*ZZ(tt-1) + Nu(tt) - Theta(tt)*II(tt) + Phi(tt)*II(tt);
        CC(tt)    = XX(tt) + ZZ(tt) + Eps(tt);
    end
    
    sim_IIWorld      = IIWorld((burnin+1):(TSIM+burnin));
    sim_II           = II(     (burnin+1):(TSIM+burnin));
    sim_Eps          = Eps(    (burnin+1):(TSIM+burnin));
    sim_Eta          = Eta(    (burnin+1):(TSIM+burnin));
    sim_Theta        = Theta(  (burnin+1):(TSIM+burnin));
    sim_Phi          = Phi(    (burnin+1):(TSIM+burnin));
    sim_Nu           = Nu(     (burnin+1):(TSIM+burnin));
    sim_XX           = XX(     (burnin+1):(TSIM+burnin));
    sim_ZZ           = ZZ(     (burnin+1):(TSIM+burnin));
    sim_CC           = CC(     (burnin+1):(TSIM+burnin));
    
    
    % put simulation output on state grid for computing consumption wealth returns
    [sim_EpsOnGrid, sim_ZZOnGrid]     = SimOutputOnStateGrid(sim_Eps, sim_ZZ, stateGridEps, stateGridZZ);
    
    % calculate return on equity(or return based on consumption claims) using simulated data from model
    [Return, CGrowth(:,sim), PDEquity]= calcReturns(sim_CC, sim_EpsOnGrid, sim_ZZOnGrid, sim_II,  PDivVecEq, stateGridII, stateGridEps, stateGridZZ, 0);
    [Rf    , CGrowth(:,sim),   PDBond]= calcReturns(sim_CC, sim_EpsOnGrid, sim_ZZOnGrid, sim_II,  PDivVecOnePerBond, stateGridII, stateGridEps, stateGridZZ, 1);
    
    MRS_1Y(:,sim)    =  exp( thetaU*log(exp(-0.034)) + (-thetaU/psiU)*log(CGrowth(2:end,sim)) + (-(1-thetaU))*log(Return));
    MRS_5Y(:,sim)    =  exp(chgfreq(log(MRS_1Y(:,sim)),5,5,5-1));
end
[mean(log(CGrowth(2:end))) std(log(CGrowth(2:end)))]
%% calculate dividend growth based on [Lettau Van Nieuwerburgh RFS]
%% eq(B1)  R_{t+1}=(D_{t+1}/D_{t})*(1+PD_{t+1})/PD_{t}
DGrowth(:,NSIM) = (Return.*PDEquity(1:end-1,NSIM))./(1+PDEquity(2:end,NSIM));
for tt =1 :size(Return,1)
    DGrowth_2(tt,1) = Return(tt)*PDEquity(tt)/(PDEquity(tt+1)+1);
end
mean(log(DGrowth))
std(log(DGrowth))
corr(log(DGrowth(2:end)),log(DGrowth(1:end-1)))
corr(log(CGrowth(2:end)),log(DGrowth(1:end)))

[mean(log(Return))  std(log(Return))]
[mean(log(Rf)) std(log(Rf))]

%% Print results 
if NoDisasters == 0
    fprintf('Disasters and the Equity Premium -Table 7 Baseline, Full Sample \n')
else
    fprintf('Disasters and the Equity Premium - Table 7 Baseline, No Disasters \n')
end
fprintf('Equity premium %3.3f \n', mean(Return-Rf))
fprintf('Risk-free rate %3.3f \n', mean(Rf)-1)

% fprintf('standard deviation of MRS \n')
% fprintf('1-Year %3.3f \n', mean(std(MRS_1Y)))
% fprintf('5-Year %3.3f \n', mean(std(MRS_5Y)))

%% Save the results
simmean_c = [mean(MRS_1Y)' mean(MRS_5Y)'];
simsd_c = [std(MRS_1Y)' std(MRS_5Y)'];
if gammaU == 6.4
    if NoDisasters  == 0
        fname = [cd, '\Results\simulation\SimNSBU\sim_NSBU2013_NSIM',num2str(NSIM),'_T',num2str(TSIM)];
    else
        fname = [cd, '\Results\simulation\SimNSBU\sim_NSBU2013_NoDisasters_NSIM',num2str(NSIM),'_T',num2str(TSIM)];
    end
    save(fname,'simmean_c','simsd_c');
elseif gammaU == 4.4
    fname = [cd, '\Results\simulation\SimNSBU\sim_NSBU2013_LowGamma_NSIM',num2str(NSIM),'_T',num2str(TSIM)];
    save(fname,'simmean_c','simsd_c');
elseif gammaU == 8.4
    fname = [cd, '\Results\simulation\SimNSBU\sim_NSBU2013_HighGamma_NSIM',num2str(NSIM),'_T',num2str(TSIM)];
    save(fname,'simmean_c','simsd_c');
end

