%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% Main code for replication of Nakamura, Steinsson, Barro adn Ursua(2013):
% Crises and Recoveries in an Empirical Model of Consumption Disasters
%
% November, 2017   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code replicates the main results of paper: Crises and Recoveries in an Empirical 
%     Model of Consumption Disasters.
%     We write this code based on the replication package by Nakamura, Steinsson, Barro 
%     and Ursua, DisastersDataAndPrograms.zip.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  ReadMe
% 1. Main code to replicate Rare Disaster model is: RepRareDisaster_NSBUPerm.m; 
%     * Specification 2 & 3 in Table 7; Baseline are computed in RepRareDisaster.m
% 
% 2. we use the estimated parameters directly from their paper. Also we only focus on the case of US. 
% 
% 3. Solve PD ratio: 
%    1) [line 153-154] 
%       nextStateFuncGen.m: generate next states for both disaster and no disaster cases.
%       All the results are saved in folder 'Results-->replication-->RepNSBU
%       You only need to run it once to save the results, later on, you can recall the results directly. No need to run it everytime.
%    2) [line 155-161]   
%       PDivs_EZW.m: solve PD ratio for both equity and one period bond with Epstein-Zin preference.
%       Results are saved in folder 'Rsults-->replication-->RepNSBU
%       You only need to run it once to save the results, later on, you can recall the results directly. No need to run it everytime.
%
%       a) PDivIntFuncGenEq.m : sub-function for equity 
%       b) PDivIntFuncGenOnePerBond.m : sub-function for bond
%  
% 6. onePerPermDis   = 1; is the case with one period permanent disaster case
%
% 7. NoDisasters -- 0: Full Sample in Table 7
%                              1: No Disaster in Table 7
% 
% N.B. : To solve for the PD ratio we use ``sigmaEps, post-1946'' and ``muPost, post-1973
%  see function ``createIntGrid(DistDropLongRun, DistDropPeakTrough)'',
% ``PDivIntFuncGenEq(currentState, intGrid,weightsOverIntGrid,nextPDivVecDis,nextPDivVecNoDis)'' and
% ``DistDisasters(nSimDistDis,maxDistDrop)''
%  THIS IS JUSTIFIED BY FOOTNOTE 31 IN THE PAPER
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
clear all; close all; clc; 
addpath('.\Library\Library_RareDisaster_NSBU','.\Results\HJBounds','.\Results\replication\RepNSBU','.\data','.\Library')
 

global maxDistDrop muPost sigmaEta sigmaNu rhoZZ  p_W p_CbW p_CbI
      
global ZZshrinkFactor fracCutoffZZ
global onePerDis onePerPermDis nIntGridEps nIntGridNu
global nSdEps nSdEta nSdPhi nSdNu  

global  rhoU  gammaU  psiU  thetaU MultiPerPermDis
global  payGrowth  pDefaultLong pDefaultShort debtEqRatio tolIntEq maxIter tolIntBond

% type of Disaster
onePerPermDis  = 1;          % 1: specification 3 & 9(PowerU=1) in Table 7
MultiPerPermDis= 0;          % 1: specification 2 in Table 7
onePerDis      = 0;          % 1: specification 10 in Table 7, we did not replicate this case

% For getDistasterDist 
NoDisasters    = 0;          %0: case with disasters
                             %1: case without disasters

nIter          = 100000 ;    % Number of iterations in calculation of PDiv
JJ             = 200;

tolIntEq       = 0.000001;   %Toleraunce for iterations on Equity integral
tolIntBond     = 0.000001;   % Tolerance for iterations on Bond integral
tolIntPerp     = 0.000001;   % Tolerance for iterations on Perpetuity integral
tolIntScaEq    = 0.000001;   % Tolerance for iterations on "scaled" Equity integral
tolIntOPEq     = 0.000001;   % Tolerance for iterations on One Period Equity integral

maxIter        = 1500;         % Maximum number of iterations

% Utility parameters
if onePerPermDis == 0 & MultiPerPermDis == 0
    gammaU     = 6.4;
elseif onePerPermDis == 1 & MultiPerPermDis == 0
    gammaU     = 3;
elseif onePerPermDis == 0 & MultiPerPermDis == 1
    gammaU     = 4.4;
end
psiU           = 2;                 
rhoU           = 0.034;           
beta           = exp(-0.034);
thetaU         = (1-gammaU)/(1-1/psiU);


% Payoff properties of long term bond
payGrowth      = 0.90;      % Gross growth rate of payoff on the perpetuity
pDefaultLong   = 0.40;      % Probability that the perpetuity will suffer a partial default in each disaster period
pDefaultShort  = 0.00;      % Probability that the one period bond will suffer a partial default in each disaster period

% Leverage on leveraged equity
debtEqRatio    = 0.50;
initPDivEq     = log(12);

% For getDistasterDist
nSimDistDis    = 1000;
maxDistDrop    = 1.0;       % max percentage drop in PeakTrough and LongRun

% State Gridsize
nStateGridEps  = 1;
nStateGridZZ   = 601;
nStateGrid     = nStateGridEps*nStateGridZZ*2;

% Integration Gridsize
nIntGridEps    = 1;
nIntGridEta    = 9;
nIntGridTheta  = 31;
nIntGridPhi    = 31;
nIntGridNu     = 1;
nIntGrid       = nIntGridEps*nIntGridEta*nIntGridTheta*nIntGridPhi*nIntGridNu;

% Grid limits in units of standard deviations
nSdEps         = 2;
nSdEta         = 3.5;
nSdTheta       = 3.5;
nSdPhi         = 3.5;
nSdNu          = 2;
fracCutoffZZ   = 0.025;

% ZZ Grid parameters
ZZshrinkFactor = 0.97;

% Parameters for simulating a "typical disaster"
nPrePeriods    = 5;
nDisPeriods    = 6;
nPostPeriods   = 25;

%% Get Parameter Estimates (we take the mean of posterior estimated parameters from paper directly, Table 1)
meanPhi        = -0.111;   sigmeanPhi     = 0.008;    % theta: a one-off permanent shift in the level of potential consumption, x_t
meanTheta      = -0.025;   sigmeanTheta = 0.007;      % phi: a shock that causes a temporary drop in consumption due to the disaster at time t

p_W            = 0.037;    sigp_W        = 0.016;     % probability of a world-wide disaster 
p_CbW          = 0.623;    sigp_CbW      = 0.076;     % probability that a particular country will enter a disaster when a world disaster begins
p_CbI          = 0.006;    sigp_CbI      = 0.003;     % probability that a particular country will enter a disaster on its own?
p_Ce           = 1-0.835;                             % probability to exit the disaster
p_CeC          = 1-p_Ce;   sigp_CeC      = 0.027;

rhoZZ          = 0.5;      sigrho_ZZ     = 0.034;     % firts order autoregressive coefficient of z_t

sigmaPhi       = 0.083;    sigsigmaPhi   = 0.006;
sigmaTheta     = 0.121;    sigsigmaTheta = 0.015;

% US country-specific estimates of parameters. See Table 3 and Table 4.
muPost         = 0.022;    sigmu         = 0.003;   % mean growth rate of potential consumption, post-1973 
sigmaEta       = 0.018;    sigsigmaEta   = 0.002;
sigmaEps       = 0.003;    sigsigmaEps   = 0.002;  % post-1946
sigmaNu        = 1e-5;       % this one is introduced to ad the convergence of the numberical algorithm
                              % set unimportant parameters to samll value, see in paper and their code

%% Generate Disasters and create grids

[DistDropBeginEnd,DistDropPeakTrough,DistDropLongRun,DistMaxZZ,DistMinZZ]...
    = DistDisasters_onePerPermDis(nSimDistDis,p_Ce,meanPhi,sigmaPhi,meanTheta,sigmaTheta);

% -----------------------------------------------------------------------------------
% This two function is the same. The reason using
% DistDisasters_onePerPermDis.m is to avoid to the issue rising by global
% varialbes
% -----------------------------------------------------------------------------------
% [DistDropBeginEnd,DistDropPeakTrough,DistDropLongRun,DistMaxZZ,DistMinZZ,DistXX,...
%     DistZZ, DistCC, medianCC, q25CC, q75CC] =DistDisasters(nSimDistDis,maxDistDrop);

if onePerPermDis == 1 | MultiPerPermDis == 1
    clear meanPhi sigmaPhi meanTheta sigmaTheta nSimDistDis sigmaEps 
    
    nSimDistDis      = 25000; % in the main rep code this is 1000;
    % State Gridsize
    nStateGridEps    = 1;
    nStateGridZZ     = 1; % in the main rep code this is 601
    nStateGrid       =  nStateGridEps*nStateGridZZ*2;
    % Integration Gridsize
    nIntGridEps      = 1;
    nIntGridEta      = 5; %9;
    nIntGridTheta    = 155; %31;
    nIntGridPhi      = 1; %31;
    nIntGridNu       = 1;
    nIntGrid         = nIntGridEps*nIntGridEta*nIntGridTheta*nIntGridPhi*nIntGridNu;
    nSdTheta         = 9.5; %3.5;
    sigmaEps         = 1e-5;
    
    if onePerPermDis == 1
        clear p_Ce
        nDisPeriods      = 1;
        p_Ce        = 1;        % footnote 30
        p_CeC       = 1-p_Ce;
    end

    meanPhi     = 0;        % No Temporary component
    sigmaPhi    = 1e-8;
    meanTheta   = mean(DistDropPeakTrough);
    sigmaTheta  = var(DistDropPeakTrough)^(1/2);
    if MultiPerPermDis == 1    
        meanTheta   = mean(DistDropPeakTrough)*p_Ce;
        sigmaTheta  = (var(DistDropPeakTrough)*p_Ce)^(1/2);
    end
end

% Generate Disaster not using baseline case parameters
if onePerPermDis == 1 | MultiPerPermDis == 1
    clear DistDropBeginEnd DistDropPeakTrough DistDropLongRun DistMaxZZ DistMinZZ
    [DistDropBeginEnd,DistDropPeakTrough,DistDropLongRun,DistMaxZZ,DistMinZZ]...
         = DistDisasters_onePerPermDis(nSimDistDis,p_Ce,meanPhi,sigmaPhi,meanTheta,sigmaTheta); %p_Ce = 1 to generate disasters
end

% Create State Grid
[stateGrid,  stateGridII, stateGridEps, stateGridZZ]   = createStateGrid_PermDis(DistMinZZ,...
    DistMaxZZ, nStateGridEps,  nStateGridZZ, nSdEps, sigmaEps);
% Create Integration grid
[intGrid, intGridEps, intGridEta, intGridTheta, intGridPhi, intGridNu, tempGridEps, tempGridTheta, tempGridPhi, ...
    tempGridEta, tempGridNu ] = createIntGrid_PermDis(DistDropLongRun, DistDropPeakTrough,...
    meanPhi,  sigmaPhi,  meanTheta, sigmaTheta, nSdTheta, nIntGridEta, nIntGridTheta, nIntGridPhi);

% Calculate weights over IntGrid
weightsOverIntGrid = calcWeights_PermDis...
(intGridEps,  intGridEta, intGridTheta, intGridPhi, intGridNu,...
 tempGridEps, tempGridTheta, tempGridPhi, tempGridEta, tempGridNu,...
 DistDropLongRun, DistDropPeakTrough, meanPhi,  sigmaPhi,  meanTheta, sigmaTheta);

%% Solve PD ratio
%  Generate nextStates for both disaster and no disaster cases
%  nextStateFuncGen (stateGrid, stateGridEps, stateGridZZ, intGrid, intGridEps, intGridEta, ...
%     intGridTheta, intGridPhi, intGridNu,0);
% % Solving PD ratio for both equity and one period bond(bill)
% 
% if onePerPermDis == 0 & MultiPerPermDis == 0
%     global p_Ce 
%     [PDivVecEq, PDivVecOnePerBond] = PDivs_EZW (stateGrid,intGrid, weightsOverIntGrid,initPDivEq);
% else
%     if onePerPermDis == 1& MultiPerPermDis == 0
%         p_Ce = 1;
%     elseif onePerPermDis == 0 & MultiPerPermDis == 1
%         clear p_Ce
%         p_Ce = 1-0.835;
%     end
%     [PDivVecEq, PDivVecOnePerBond] = PDivs_EZW_PermDis(stateGrid,intGrid, weightsOverIntGrid,initPDivEq, p_Ce);
% end

% if onePerPermDis == 1 & MultiPerPermDis == 0
%     fname = [cd, '\Results\simulation\SimNSBU\logPDivRatio_onePerPermDis'];
% elseif onePerPermDis == 0 & MultiPerPermDis == 1
%     fname = [cd, '\Results\replication\RepNSBU\logPDivRatio_MultiPerPermDis']; 
% end
% save(fname,'PDivVecEq','PDivVecOnePerBond')


if onePerPermDis == 1 & MultiPerPermDis == 0
    logPDivRatio= [cd, '\Results\simulation\SimNSBU\logPDivRatio_onePerPermDis'];
elseif onePerPermDis == 0 & MultiPerPermDis == 1
    logPDivRatio= [cd, '\Results\simulation\SimNSBU\logPDivRatio_MultiPerPermDis'];
end
load(logPDivRatio); %#check size(stateGrid,1) == size(PDivVecEq) == size(PDivVecOnePerBond)


%% Model simulation
TSIM =  600000/4;%;1000200000
NSIM =  1; %10;
burnin= 240;
rand('seed', 17631)
randn('seed', 17631)

% % % TSIM =  floor(742/12)+1;%150;% reported results in Favero, Ortu, Tamoni and Yang
% % % NSIM =  1000;%1000; %100000;
% % % burnin= 240;
% % % rand('seed', 1761)  
% % % randn('seed', 1761) 

if onePerPermDis == 1& MultiPerPermDis == 0
    p_Ce = 1;
elseif onePerPermDis == 0 & MultiPerPermDis == 1
    clear p_Ce
    p_Ce = 1-0.835;
end

if onePerPermDis == 1 
    ThetaDist = DistDropPeakTrough;
end

for sim = 1:NSIM
    
    sim
    
    % Set initial values
    II          =  zeros(TSIM+burnin,1);
    IIWorld     =  zeros(TSIM+burnin,1);
    XX          =  zeros(TSIM+burnin,1);
    ZZ          =  zeros(TSIM+burnin,1);
    Eps         =  zeros(TSIM+burnin,1);
    CC          =  zeros(TSIM+burnin,1);
    Eta         =  zeros(TSIM+burnin,1);
    Phi         =  zeros(TSIM+burnin,1);
    Theta       =  zeros(TSIM+burnin,1);
    Nu          =  zeros(TSIM+burnin,1);
    
    XX(1)       = 1;
    CC(1)       = XX(1) + ZZ(1) + Eps(1);

    % Simulate model
    for tt = 2:TSIM+burnin
        IIWorld(tt)         = binornd(1,p_W);
        
        if  II(tt-1) == 1
            temppp = 1-p_Ce;      %ppC(2)
        else
            if IIWorld(tt) == 1
                temppp = p_CbW;  %ppC(3)
            else
                temppp = p_CbI;  %ppC(1)
            end
        end
 
        if NoDisasters ~=0
            II(tt)    = 0;
        else
            II(tt)    = binornd(1,temppp);
        end
        
        if II(tt) == 1
            if  onePerPermDis==0 
                Theta(tt) = meanTheta+sigmaTheta*randn(1);
            elseif onePerPermDis == 1 & MultiPerPermDis == 0
                tempXX    = unifrnd(0,1);
                tempXX    = max(1,round(tempXX*length(ThetaDist)));
                Theta(tt) = ThetaDist(tempXX);
            end
        end
        
        if onePerPermDis == 0 & MultiPerPermDis == 0
            k = 1;
            while k < 2
                rdnumber = meanPhi + sigmaPhi*randn(1,1);
                if rdnumber < 0 & rdnumber> -1000
                    Phi(tt) = rdnumber; % truncated normal variable
                    k = k +1;
                else
                    k = k;
                end
            end
        else
            Phi(tt) = Theta(tt);
        end
        
        Eps(tt)   = sigmaEps*randn(1);
        Eta(tt)   = sigmaEta*randn(1);
        Nu(tt)    = sigmaNu *randn(1);
        
        XX(tt)    = XX(tt-1) + muPost + Eta(tt) + Theta(tt)*II(tt);
        ZZ(tt)    = rhoZZ*ZZ(tt-1) + Nu(tt) - Theta(tt)*II(tt) + Phi(tt)*II(tt);
        CC(tt)    = XX(tt) + ZZ(tt) + Eps(tt);
    end
    
    sim_IIWorld      = IIWorld((burnin+1):(TSIM+burnin));
    sim_II           = II(     (burnin+1):(TSIM+burnin));
    sim_Eps          = Eps(    (burnin+1):(TSIM+burnin));
    sim_Eta          = Eta(    (burnin+1):(TSIM+burnin));
    sim_Theta        = Theta(  (burnin+1):(TSIM+burnin));
    sim_Phi          = Phi(    (burnin+1):(TSIM+burnin));
    sim_Nu           = Nu(     (burnin+1):(TSIM+burnin));
    sim_XX           = XX(     (burnin+1):(TSIM+burnin));
    sim_ZZ           = ZZ(     (burnin+1):(TSIM+burnin));
    sim_CC           = CC(     (burnin+1):(TSIM+burnin));
    
    
    % put simulation output on state grid for computing consumption wealth returns
    [sim_EpsOnGrid, sim_ZZOnGrid]     = SimOutputOnStateGrid(sim_Eps, sim_ZZ, stateGridEps, stateGridZZ);
    
    % calculate return on equity(or return based on consumption claims) using simulated data from model
    [Return, CGrowth(:,sim), PDEquity]= calcReturns(sim_CC, sim_EpsOnGrid, sim_ZZOnGrid, sim_II,  PDivVecEq, stateGridII, stateGridEps, stateGridZZ, 0);
    [Rf    , CGrowth(:,sim),   PDBond]= calcReturns(sim_CC, sim_EpsOnGrid, sim_ZZOnGrid, sim_II,  PDivVecOnePerBond, stateGridII, stateGridEps, stateGridZZ, 1);
    
    MRS_1Y(:,sim)    =  exp( thetaU*log(exp(-0.034)) + (-thetaU/psiU)*log(CGrowth(2:end,sim)) + (-(1-thetaU))*log(Return));
    MRS_5Y(:,sim)    =  exp(chgfreq(log(MRS_1Y(:,sim)),5,5,5-1));
    
    DisasterII(:,sim) = sim_II;
    DisasterZZ(:,sim) = sim_ZZ;
end
 
[mean(log(CGrowth(2:end))) std(log(CGrowth(2:end)))]

[mean(log(Return))  std(log(Return))]
[mean(log(Rf)) std(log(Rf))]

%% Save results
simmean_c = [mean(MRS_1Y)' mean(MRS_5Y)'];
simsd_c = [std(MRS_1Y)' std(MRS_5Y)'];
if onePerPermDis == 1
    fname = [cd, '\Results\simulation\SimNSBU\sim_NSBU2013_onePerPermDis_NSIM',num2str(NSIM),'_T',num2str(TSIM)];
    save(fname,'simmean_c','simsd_c');
elseif onePerPermDis == 0 & MultiPerPermDis == 1
    fname = [cd, '\Results\simulation\SimNSBU\sim_NSBU2013_MultiPerPermDis_NSIM',num2str(NSIM),'_T',num2str(TSIM)];
    save(fname,'simmean_c','simsd_c');
end

if NoDisasters == 0
    if onePerPermDis == 0
        fprintf('Permanent Disasters and the Equity Premium - Permanent, Full Sample \n')
    else
        fprintf('Permanent and One-period Disasters and the Equity Premium - Permanent and One Period, Full Sample \n')
    end
else
    if onePerPermDis == 0
        fprintf('Permanent Disasters and the Equity Premium - Permanent, No disasters\n')
    else
        fprintf('Permanent and One-period  Disasters and the Equity Premium - Permanent and One Period, No disasters\n')
    end
end
fprintf('Equity premium %3.3f \n', mean(Return-Rf))
fprintf('Risk-free rate %3.3f \n', mean(Rf)-1)

% fprintf('standard deviation of MRS \n')
% fprintf( '%11s %5s\n','mean','std');
% fprintf('1-Year %3.3f %3.3f \n', [mean(mean(MRS_1Y)) mean(std(MRS_1Y))])
% % fprintf('5-Year %3.3f %3.3f \n', [mean(mean(MRS_5Y(:,1:9))) mean(std(MRS_5Y(:,1:9)))])
% fprintf('5-Year %3.3f %3.3f \n', [mean(mean(MRS_5Y)) mean(std(MRS_5Y))])




